@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: actions.c 8205 2009-04-10 17:44:17Z charles $
  */
 
 #include <string.h>
@@ -44,20 +44,13 @@ action_cb( GtkAction * a,
 
 static GtkRadioActionEntry sort_radio_entries[] =
 {
-    { "sort-by-activity", NULL,      N_( "Sort by _Activity" ),        NULL,
-      NULL, 0 },
-    { "sort-by-name",     NULL,        N_( "Sort by _Name" ),     NULL,
-      NULL, 1 },
-    { "sort-by-progress", NULL,        N_( "Sort by _Progress" ), NULL,
-      NULL, 2 },
-    { "sort-by-ratio",    NULL,        N_( "Sort by _Ratio" ),    NULL,
-      NULL, 3 },
-    { "sort-by-state",    NULL,        N_( "Sort by _State" ),    NULL,
-      NULL, 4 },
-    { "sort-by-tracker",  NULL,        N_( "Sort by _Tracker" ),  NULL,
-      NULL, 5 },
-    { "sort-by-age",      NULL,        N_( "Sort by A_ge" ),      NULL,
-      NULL, 6 }
+    { "sort-by-activity", NULL, N_( "Sort by _Activity" ), NULL, NULL, 0 },
+    { "sort-by-name",     NULL, N_( "Sort by _Name" ),     NULL, NULL, 1 },
+    { "sort-by-progress", NULL, N_( "Sort by _Progress" ), NULL, NULL, 2 },
+    { "sort-by-ratio",    NULL, N_( "Sort by _Ratio" ),    NULL, NULL, 3 },
+    { "sort-by-state",    NULL, N_( "Sort by _State" ),    NULL, NULL, 4 },
+    { "sort-by-tracker",  NULL, N_( "Sort by _Tracker" ),  NULL, NULL, 5 },
+    { "sort-by-age",      NULL, N_( "Sort by A_ge" ),      NULL, NULL, 6 }
 };
 
 static void
@@ -74,10 +67,8 @@ sort_changed_cb( GtkAction            * action UNUSED,
 
 static GtkToggleActionEntry show_toggle_entries[] =
 {
-    { "toggle-main-window", NULL,
-      N_( "_Main Window" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
-    { "toggle-message-log", NULL,
-      N_( "Message _Log" ), NULL, NULL, G_CALLBACK( action_cb ), FALSE }
+    { "toggle-main-window", NULL, N_( "_Main Window" ), NULL, NULL, G_CALLBACK( action_cb ), TRUE },
+    { "toggle-message-log", NULL, N_( "Message _Log" ), NULL, NULL, G_CALLBACK( action_cb ), FALSE }
 };
 
 static void
@@ -120,6 +111,7 @@ static GtkActionEntry        entries[] =
     { "verify-torrent", NULL, N_( "_Verify Local Data" ), NULL, NULL, G_CALLBACK( action_cb ) },
     { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
+    { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
     { "remove-torrent", GTK_STOCK_REMOVE, NULL, "Delete", N_( "Remove torrent" ), G_CALLBACK( action_cb ) },
     { "delete-torrent", GTK_STOCK_DELETE, N_( "_Delete Files and Remove" ), "<shift>Delete", NULL, G_CALLBACK( action_cb ) },
     { "new-torrent", GTK_STOCK_NEW, N_( "_New..." ), NULL, N_( "Create a torrent" ), G_CALLBACK( action_cb ) },
