@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.c 6862 2008-10-07 15:17:52Z charles $
+ * $Id: conf.c 8205 2009-04-10 17:44:17Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -39,20 +39,23 @@
 #include <libtransmission/platform.h>
 
 #include "conf.h"
+#include "tr-prefs.h"
 #include "util.h"
 
+#define MY_NAME "transmission"
+
 static char * gl_confdir = NULL;
 static char * gl_lockpath = NULL;
 
 /* errstr may be NULL, this might be called before GTK is initialized */
 gboolean
-cf_init( const char *dir,
-         char **     errstr )
+cf_init( const char   * configDir,
+         char        ** errstr )
 {
     if( errstr != NULL )
         *errstr = NULL;
 
-    gl_confdir = g_strdup( dir );
+    gl_confdir = g_strdup( configDir );
 
     if( mkdir_p( gl_confdir, 0755 ) )
         return TRUE;
@@ -141,6 +144,71 @@ cf_lock( tr_lockfile_state_t *tr_state,
 ****
 ***/
 
+static void cf_check_older_configs( void );
+
+/**
+ * This is where we initialize the preferences file with the default values.
+ * If you add a new preferences key, you /must/ add a default value here.
+ */
+static void
+tr_prefs_init_defaults( tr_benc * d )
+{
+    const char * str;
+
+    cf_check_older_configs( );
+
+#ifdef HAVE_GIO
+    str = NULL;
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DOWNLOAD );
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DESKTOP );
+    if( !str ) str = tr_getDefaultDownloadDir( );
+    tr_bencDictAddStr( d, PREF_KEY_DIR_WATCH, str );
+    tr_bencDictAddInt( d, PREF_KEY_DIR_WATCH_ENABLED, FALSE );
+#endif
+
+    tr_bencDictAddInt( d, PREF_KEY_INHIBIT_HIBERNATION, FALSE );
+    tr_bencDictAddInt( d, PREF_KEY_BLOCKLIST_UPDATES_ENABLED, TRUE );
+
+    tr_bencDictAddStr( d, PREF_KEY_OPEN_DIALOG_FOLDER, g_get_home_dir( ) );
+
+    tr_bencDictAddInt( d, PREF_KEY_TOOLBAR, TRUE );
+    tr_bencDictAddInt( d, PREF_KEY_FILTERBAR, TRUE );
+    tr_bencDictAddInt( d, PREF_KEY_STATUSBAR, TRUE );
+    tr_bencDictAddInt( d, PREF_KEY_SHOW_TRAY_ICON, FALSE );
+    tr_bencDictAddInt( d, PREF_KEY_SHOW_DESKTOP_NOTIFICATION, TRUE );
+    tr_bencDictAddStr( d, PREF_KEY_STATUSBAR_STATS, "total-ratio" );
+
+    tr_bencDictAddInt( d, PREF_KEY_SCHED_LIMIT_ENABLED, FALSE );
+    tr_bencDictAddInt( d, PREF_KEY_SCHED_BEGIN,    60 * 23 ); /* 11pm */
+    tr_bencDictAddInt( d, PREF_KEY_SCHED_END,      60 * 7 );  /* 7am */
+    tr_bencDictAddInt( d, PREF_KEY_SCHED_DL_LIMIT, 200 );   /* 2x the other limit */
+    tr_bencDictAddInt( d, PREF_KEY_SCHED_UL_LIMIT, 100 );   /* 2x the other limit */
+
+    tr_bencDictAddInt( d, PREF_KEY_OPTIONS_PROMPT, TRUE );
+
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_HEIGHT, 500 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_WIDTH, 300 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_X, 50 );
+    tr_bencDictAddInt( d, PREF_KEY_MAIN_WINDOW_Y, 50 );
+    tr_bencDictAddStr( d, PREF_KEY_MAIN_WINDOW_LAYOUT_ORDER, "menu,toolbar,filter,list,statusbar" );
+
+    str = NULL;
+#if GLIB_CHECK_VERSION( 2, 14, 0 )
+    if( !str ) str = g_get_user_special_dir( G_USER_DIRECTORY_DOWNLOAD );
+#endif
+    if( !str ) str = tr_getDefaultDownloadDir( );
+    tr_bencDictAddStr( d, TR_PREFS_KEY_DOWNLOAD_DIR, str );
+
+    tr_bencDictAddInt( d, PREF_KEY_ASKQUIT, TRUE );
+
+    tr_bencDictAddStr( d, PREF_KEY_SORT_MODE, "sort-by-name" );
+    tr_bencDictAddInt( d, PREF_KEY_SORT_REVERSED, FALSE );
+    tr_bencDictAddInt( d, PREF_KEY_MINIMAL_VIEW, FALSE );
+
+    tr_bencDictAddInt( d, PREF_KEY_START, TRUE );
+    tr_bencDictAddInt( d, PREF_KEY_TRASH_ORIGINAL, FALSE );
+}
+
 static char*
 getPrefsFilename( void )
 {
@@ -151,25 +219,30 @@ getPrefsFilename( void )
 static tr_benc*
 getPrefs( void )
 {
-    static tr_benc  dict;
+    static tr_benc settings;
     static gboolean loaded = FALSE;
 
     if( !loaded )
     {
-        char * filename = getPrefsFilename( );
-        if( tr_bencLoadJSONFile( filename, &dict ) )
-            tr_bencInitDict( &dict, 100 );
-        g_free( filename );
+        tr_bencInitDict( &settings, 0 );
+        tr_prefs_init_defaults( &settings );
+        tr_sessionLoadSettings( &settings, gl_confdir, MY_NAME );
         loaded = TRUE;
     }
 
-    return &dict;
+    return &settings;
 }
 
 /***
 ****
 ***/
 
+tr_benc*
+pref_get_all( void )
+{
+    return getPrefs( );
+}
+
 int64_t
 pref_int_get( const char * key )
 {
@@ -183,18 +256,7 @@ void
 pref_int_set( const char * key,
               int64_t      value )
 {
-    tr_benc * d = getPrefs( );
-
-    tr_bencDictRemove( d, key );
-    tr_bencDictAddInt( d, key, value );
-}
-
-void
-pref_int_set_default( const char * key,
-                      int64_t      value )
-{
-    if( !tr_bencDictFind( getPrefs( ), key ) )
-        pref_int_set( key, value );
+    tr_bencDictAddInt( getPrefs( ), key, value );
 }
 
 /***
@@ -234,13 +296,6 @@ pref_flag_set( const char * key,
     pref_int_set( key, value != 0 );
 }
 
-void
-pref_flag_set_default( const char * key,
-                       gboolean     value )
-{
-    pref_int_set_default( key, value != 0 );
-}
-
 /***
 ****
 ***/
@@ -249,27 +304,14 @@ const char*
 pref_string_get( const char * key )
 {
     const char * str = NULL;
-
     tr_bencDictFindStr( getPrefs( ), key, &str );
     return str;
 }
 
 void
-pref_string_set( const char * key,
-                 const char * value )
+pref_string_set( const char * key, const char * value )
 {
-    tr_benc * d = getPrefs( );
-
-    tr_bencDictRemove( d, key );
-    tr_bencDictAddStr( d, key, value );
-}
-
-void
-pref_string_set_default( const char * key,
-                         const char * value )
-{
-    if( !tr_bencDictFind( getPrefs( ), key ) )
-        pref_string_set( key, value );
+    tr_bencDictAddStr( getPrefs( ), key, value );
 }
 
 /***
@@ -277,16 +319,9 @@ pref_string_set_default( const char * key,
 ***/
 
 void
-pref_save( void )
+pref_save( tr_session * session )
 {
-    char * filename = getPrefsFilename( );
-    char * path = g_path_get_dirname( filename );
-
-    mkdir_p( path, 0755 );
-    tr_bencSaveJSONFile( filename, getPrefs( ) );
-
-    g_free( path );
-    g_free( filename );
+    tr_sessionSaveSettings( session, gl_confdir, getPrefs( ) );
 }
 
 /***
@@ -316,32 +351,29 @@ static char*
 getCompat080PrefsFilename( void )
 {
     assert( gl_confdir != NULL );
-    return g_build_filename(
-               g_get_home_dir( ), ".transmission", "gtk", "prefs", NULL );
+
+    return g_build_filename( g_get_home_dir( ), ".transmission", "gtk", "prefs", NULL );
 }
 
 static char*
 getCompat090PrefsFilename( void )
 {
     assert( gl_confdir != NULL );
-    return g_build_filename(
-               g_get_home_dir( ), ".transmission", "gtk", "prefs.ini", NULL );
+
+    return g_build_filename( g_get_home_dir( ), ".transmission", "gtk", "prefs.ini", NULL );
 }
 
 static char*
 getCompat121PrefsFilename( void )
 {
-    return g_build_filename(
-               g_get_user_config_dir( ), "transmission", "gtk", "prefs.ini",
-               NULL );
+    return g_build_filename( g_get_user_config_dir( ), "transmission", "gtk", "prefs.ini", NULL );
 }
 
 static void
 translate_08_to_09( const char* oldfile,
                     const char* newfile )
 {
-    static struct pref_entry
-    {
+    static struct pref_entry {
         const char*   oldkey;
         const char*   newkey;
     } pref_table[] = {
@@ -369,7 +401,7 @@ translate_08_to_09( const char* oldfile,
 
     if( g_file_get_contents( oldfile, &contents, &contents_len, NULL )
       && !tr_bencLoad( contents, contents_len, &top, NULL )
-      && top.type == TYPE_DICT )
+      && tr_bencIsDict( &top ) )
     {
         unsigned int i;
         g_string_append( out, "\n[general]\n" );
@@ -456,7 +488,7 @@ translate_keyfile_to_json( const char * old_file,
     tr_bencFree( &dict );
 }
 
-void
+static void
 cf_check_older_configs( void )
 {
     char * filename = getPrefsFilename( );
@@ -469,18 +501,19 @@ cf_check_older_configs( void )
 
         if( g_file_test( key1, G_FILE_TEST_IS_REGULAR ) )
         {
+            g_message( _( "Importing \"%s\"" ), key1 );
             translate_keyfile_to_json( key1, filename );
         }
         else if( g_file_test( key2, G_FILE_TEST_IS_REGULAR ) )
         {
+            g_message( _( "Importing \"%s\"" ), key2 );
             translate_keyfile_to_json( key2, filename );
         }
         else if( g_file_test( benc, G_FILE_TEST_IS_REGULAR ) )
         {
             char * tmpfile;
-            int    fd =
-                g_file_open_tmp( "transmission-prefs-XXXXXX", &tmpfile,
-                                 NULL );
+            int    fd = g_file_open_tmp( "transmission-prefs-XXXXXX", &tmpfile, NULL );
+            g_message( _( "Importing \"%s\"" ), benc );
             if( fd != -1 ) close( fd );
             translate_08_to_09( benc, tmpfile );
             translate_keyfile_to_json( tmpfile, filename );
@@ -494,4 +527,3 @@ cf_check_older_configs( void )
 
     g_free( filename );
 }
-
