@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 7767 2009-01-20 18:31:37Z charles $
+ * $Id: main.c 8205 2009-04-10 17:44:17Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -35,12 +35,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
-#include <gdk/gdk.h>
-#ifdef GDK_WINDOWING_X11
- #include <X11/Xatom.h>
- #include <gdk/gdkx.h>
-#endif
-
 #include <libtransmission/transmission.h>
 #include <libtransmission/rpcimpl.h>
 #include <libtransmission/utils.h>
@@ -208,7 +202,7 @@ accumulateCanUpdateForeach( GtkTreeModel *      model,
 static void
 refreshTorrentActions( struct cbdata * data )
 {
-    int                canUpdate;
+    int canUpdate;
     struct counts_data counts;
     GtkTreeSelection * s = data->sel;
 
@@ -231,15 +225,17 @@ refreshTorrentActions( struct cbdata * data )
     {
         GtkTreeView *  view = gtk_tree_selection_get_tree_view( s );
         GtkTreeModel * model = gtk_tree_view_get_model( view );
-        const int  torrentCount = gtk_tree_model_iter_n_children( model, NULL ) != 0;
+        const int torrentCount = gtk_tree_model_iter_n_children( model, NULL ) != 0;
         action_sensitize( "select-all", torrentCount != 0 );
         action_sensitize( "deselect-all", torrentCount != 0 );
     }
 
     {
         tr_session * session = tr_core_session( data->core );
-        int activeCount = tr_sessionGetActiveTorrentCount( session );
-        action_sensitize( "pause-all-torrents", activeCount != 0 );
+        const int active = tr_sessionGetActiveTorrentCount( session );
+        const int total = tr_sessionCountTorrents( session );
+        action_sensitize( "pause-all-torrents", active != 0 );
+        action_sensitize( "start-all-torrents", active != total );
     }
 }
 
@@ -350,7 +346,7 @@ main( int     argc,
     gboolean            showversion = FALSE;
     gboolean            startpaused = FALSE;
     gboolean            startminimized = FALSE;
-    char *              domain = MY_NAME;
+    const char *        domain = MY_NAME;
     char *              configDir = NULL;
     tr_lockfile_state_t tr_state;
 
@@ -386,7 +382,7 @@ main( int     argc,
 
     gerr = NULL;
     if( !gtk_init_with_args( &argc, &argv, _( "[torrent files]" ), entries,
-                             domain, &gerr ) )
+                             (char*)domain, &gerr ) )
     {
         fprintf( stderr, "%s\n", gerr->message );
         g_clear_error( &gerr );
@@ -404,7 +400,6 @@ main( int     argc,
 
     tr_notify_init( );
     didinit = cf_init( configDir, NULL ); /* must come before actions_init */
-    tr_prefs_init_global( );
 
     myUIManager = gtk_ui_manager_new ( );
     actions_init ( myUIManager, cbdata );
@@ -436,9 +431,16 @@ main( int     argc,
 
     if( didlock && ( didinit || cf_init( configDir, &err ) ) )
     {
+        const char * str;
         GtkWindow * win;
         tr_session * session;
 
+        /* ensure the directories are created */
+       if(( str = pref_string_get( PREF_KEY_DIR_WATCH )))
+           mkdir_p( str, 0777 );
+       if(( str = pref_string_get( TR_PREFS_KEY_DOWNLOAD_DIR )))
+           mkdir_p( str, 0777 );
+
         /* initialize the libtransmission session */
         session = tr_sessionInit( "gtk", configDir, TRUE, pref_get_all( ) );
         cbdata->core = tr_core_new( session );
@@ -891,9 +893,9 @@ setupdrag( GtkWidget *    widget,
            struct cbdata *data )
 {
     GtkTargetEntry targets[] = {
-        { "STRING",          0, 0 },
-        { "text/plain",      0, 0 },
-        { "text/uri-list",   0, 0 },
+        { (char*)"STRING",          0, 0 },
+        { (char*)"text/plain",      0, 0 },
+        { (char*)"text/uri-list",   0, 0 },
     };
 
     g_signal_connect( widget, "drag_data_received", G_CALLBACK(
@@ -1043,44 +1045,37 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_PEER_PORT ) )
     {
-        const int port = pref_int_get( key );
-        tr_sessionSetPeerPort( tr, port );
+        tr_sessionSetPeerPort( tr, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_BLOCKLIST_ENABLED ) )
     {
-        const gboolean flag = pref_flag_get( key );
-        tr_blocklistSetEnabled( tr, flag );
+        tr_blocklistSetEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, PREF_KEY_SHOW_TRAY_ICON ) )
     {
         const int show = pref_flag_get( key );
         if( show && !cbdata->icon )
             cbdata->icon = tr_icon_new( cbdata->core );
-        else if( !show && cbdata->icon )
-        {
+        else if( !show && cbdata->icon ) {
             g_object_unref( cbdata->icon );
             cbdata->icon = NULL;
         }
     }
     else if( !strcmp( key, TR_PREFS_KEY_DSPEED_ENABLED ) )
     {
-        const gboolean b = pref_flag_get( key );
-        tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, b );
+        tr_sessionSetSpeedLimitEnabled( tr, TR_DOWN, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_DSPEED ) )
     {
-        const int limit = pref_int_get( key );
-        tr_sessionSetSpeedLimit( tr, TR_DOWN, limit );
+        tr_sessionSetSpeedLimit( tr, TR_DOWN, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_USPEED_ENABLED ) )
     {
-        const gboolean b = pref_flag_get( key );
-        tr_sessionSetSpeedLimitEnabled( tr, TR_UP, b );
+        tr_sessionSetSpeedLimitEnabled( tr, TR_UP, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_USPEED ) )
     {
-        const int limit = pref_int_get( key );
-        tr_sessionSetSpeedLimit( tr, TR_UP, limit );
+        tr_sessionSetSpeedLimit( tr, TR_UP, pref_int_get( key ) );
     }
     else if( !strncmp( key, "sched-", 6 ) )
     {
@@ -1104,8 +1099,7 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_WHITELIST ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCWhitelist( tr, s );
+        tr_sessionSetRPCWhitelist( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_WHITELIST_ENABLED ) )
     {
@@ -1113,48 +1107,39 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_USERNAME ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCUsername( tr, s );
+        tr_sessionSetRPCUsername( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_PASSWORD ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetRPCPassword( tr, s );
+        tr_sessionSetRPCPassword( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_AUTH_REQUIRED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetRPCPasswordEnabled( tr, enabled );
+        tr_sessionSetRPCPasswordEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxy( tr, s );
+        tr_sessionSetProxy( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_TYPE ) )
     {
-        const int i = pref_int_get( key );
-        tr_sessionSetProxyType( tr, i );
+        tr_sessionSetProxyType( tr, pref_int_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_ENABLED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetProxyEnabled( tr, enabled );
+        tr_sessionSetProxyEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_AUTH_ENABLED ) )
     {
-        const gboolean enabled = pref_flag_get( key );
-        tr_sessionSetProxyAuthEnabled( tr, enabled );
+        tr_sessionSetProxyAuthEnabled( tr, pref_flag_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_USERNAME ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyUsername( tr, s );
+        tr_sessionSetProxyUsername( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_PASSWORD ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyPassword( tr, s );
+        tr_sessionSetProxyPassword( tr, pref_string_get( key ) );
     }
     else if( !strcmp( key, TR_PREFS_KEY_PROXY_PORT ) )
     {
@@ -1162,8 +1147,7 @@ prefschanged( TrCore * core UNUSED,
     }
     else if( !strcmp( key, TR_PREFS_KEY_RPC_PASSWORD ) )
     {
-        const char * s = pref_string_get( key );
-        tr_sessionSetProxyPassword( tr, s );
+        tr_sessionSetProxyPassword( tr, pref_string_get( key ) );
     }
 }
 
@@ -1375,6 +1359,14 @@ removeSelected( struct cbdata * data,
     }
 }
 
+static void
+startAllTorrents( struct cbdata * data )
+{
+    tr_session * session = tr_core_session( data->core );
+    const char * cmd = "{ \"method\": \"torrent-start\" }";
+    tr_rpc_request_exec_json( session, cmd, strlen( cmd ), NULL, NULL );
+}
+
 static void
 pauseAllTorrents( struct cbdata * data )
 {
@@ -1409,6 +1401,10 @@ doAction( const char * action_name, gpointer user_data )
     {
         pauseAllTorrents( data );
     }
+    else if( !strcmp( action_name, "start-all-torrents" ) )
+    {
+        startAllTorrents( data );
+    }
     else if( !strcmp( action_name, "pause-torrent" ) )
     {
         GtkTreeSelection * s = tr_window_get_selection( data->wind );
