@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-window.c 7260 2008-12-03 15:59:51Z charles $
+ * $Id: tr-window.c 8205 2009-04-10 17:44:17Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -92,14 +92,13 @@ typedef struct
     filter_mode_t         filter_mode;
     filter_text_mode_t    filter_text_mode;
     char *                filter_text;
+    GtkToggleButton     * filter_toggles[FILTER_MODE_QTY];
 }
 PrivateData;
 
 #define PRIVATE_DATA_KEY "private-data"
 
-#define FILTER_MODE_KEY "tr-filter-mode"
 #define FILTER_TEXT_MODE_KEY "tr-filter-text-mode"
-#define FILTER_TOGGLES_KEY "tr-filter-toggles"
 
 static PrivateData*
 get_private_data( TrWindow * w )
@@ -128,7 +127,7 @@ view_row_activated( GtkTreeView       * tree_view UNUSED,
                     GtkTreeViewColumn * column    UNUSED,
                     gpointer            user_data UNUSED )
 {
-    action_activate( "show-torrent-details" );
+    action_activate( "show-torrent-properties" );
 }
 
 static gboolean is_row_visible( GtkTreeModel *,
@@ -147,15 +146,20 @@ makeview( PrivateData * p,
 
     view = gtk_tree_view_new( );
     gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( view ), FALSE );
+    gtk_tree_view_set_fixed_height_mode( GTK_TREE_VIEW( view ), TRUE );
 
     p->selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
 
+    p->column = col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,		      
+        "title", _("Torrent"),
+        "resizable", TRUE,
+        "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
+        NULL));
+
     p->renderer = r = torrent_cell_renderer_new( );
-    p->column = col = gtk_tree_view_column_new_with_attributes(
-                    _( "Torrent" ), r, "torrent", MC_TORRENT_RAW, NULL );
-    g_object_set( G_OBJECT( col ), "resizable", TRUE,
-                  "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-                  NULL );
+    gtk_tree_view_column_pack_start( col, r, FALSE );
+    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT_RAW );
+    
     gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
     g_object_set( r, "xpad", GUI_PAD_SMALL, "ypad", GUI_PAD_SMALL, NULL );
 
@@ -198,8 +202,10 @@ prefsChanged( TrCore * core UNUSED,
     if( !strcmp( key, PREF_KEY_MINIMAL_VIEW ) )
     {
         g_object_set( p->renderer, "minimal", pref_flag_get( key ), NULL );
-        gtk_tree_view_column_queue_resize( p->column );
-        gtk_widget_queue_draw( p->view );
+        /* since the cell size has changed, we need gtktreeview to revalidate
+         * its fixed-height mode values.  Unfortunately there's not an API call
+         * for that, but it *does* revalidate when it thinks the style's been tweaked */
+        g_signal_emit_by_name( p->view, "style-set", NULL, NULL );
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR ) )
     {
@@ -332,7 +338,9 @@ checkFilterMode( filter_mode_t filter_mode,
         case FILTER_MODE_ACTIVE:
         {
             const tr_stat * s = tr_torrentStatCached( tor );
-            ret = s->peersSendingToUs > 0 || s->peersGettingFromUs > 0;
+            ret = s->peersSendingToUs > 0
+               || s->peersGettingFromUs > 0
+               || tr_torrentGetActivity( tor ) == TR_STATUS_CHECK;
             break;
         }
 
@@ -385,34 +393,23 @@ static void
 filter_toggled_cb( GtkToggleButton * toggle,
                    gpointer          vprivate )
 {
-    PrivateData *       p = vprivate;
-    GSList *            l;
-    GSList *            toggles = g_object_get_data( G_OBJECT(
-                                                         toggle ),
-                                                     FILTER_TOGGLES_KEY );
-    const gboolean      isActive = gtk_toggle_button_get_active( toggle );
-    const filter_mode_t mode =
-        GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle ),
-                                             FILTER_MODE_KEY ) );
+    int mode = -1;
+    PrivateData * p = vprivate;
 
-    /* update the filter */
-    if( isActive )
+    if( gtk_toggle_button_get_active( toggle ) )
     {
+        int i;
+        for( i=0; i<FILTER_MODE_QTY; ++i )
+            if( p->filter_toggles[i] == toggle )
+                mode = i;
+            else
+                gtk_toggle_button_set_active( p->filter_toggles[i], FALSE );
+    }
+
+    if( mode >= 0 ) {
         p->filter_mode = mode;
         refilter( p );
     }
-
-    /* deactivate the other toggles */
-    for( l = toggles; l != NULL; l = l->next )
-    {
-        GtkToggleButton * walk = GTK_TOGGLE_BUTTON( l->data );
-        if( isActive && ( toggle != walk ) )
-            gtk_toggle_button_set_active( walk, FALSE );
-    }
-
-    /* at least one button must always be set */
-    if( !isActive && ( p->filter_mode == mode ) )
-        gtk_toggle_button_set_active( toggle, TRUE );
 }
 
 static void
@@ -498,16 +495,16 @@ onAskTrackerQueryTooltip( GtkWidget *            widget UNUSED,
 ***/
 
 GtkWidget *
-tr_window_new( GtkUIManager * ui_manager,
-               TrCore *       core )
+tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 {
     int           i, n;
     const char *  pch;
     PrivateData * p;
+    GtkWidget   *mainmenu, *toolbar, *filter, *list, *status;
     GtkWidget *   vbox, *w, *self, *h, *c, *s, *image, *menu;
     GtkWindow *   win;
     GSList *      l;
-    GSList *      toggles;
+
     const char *  filter_names[FILTER_MODE_QTY] = {
         /* show all torrents */
         N_( "A_ll" ),
@@ -537,20 +534,18 @@ tr_window_new( GtkUIManager * ui_manager,
     gtk_window_set_title( win, g_get_application_name( ) );
     gtk_window_set_role( win, "tr-main" );
     gtk_window_set_default_size( win,
-                                pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
-                                pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
+                                 pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
+                                 pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
     gtk_window_move( win, pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
-                    pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
-    gtk_window_add_accel_group( win,
-                               gtk_ui_manager_get_accel_group( ui_manager ) );
+                     pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+    gtk_window_add_accel_group( win, gtk_ui_manager_get_accel_group( ui_mgr ) );
 
     /* window's main container */
     vbox = gtk_vbox_new ( FALSE, 0 );
     gtk_container_add ( GTK_CONTAINER( self ), vbox );
 
     /* main menu */
-    w = action_get_widget( "/main-window-menu" );
-    gtk_box_pack_start( GTK_BOX( vbox ), w, FALSE, FALSE, 0 );
+    w = mainmenu = action_get_widget( "/main-window-menu" );
     w = action_get_widget( "/main-window-menu/torrent-menu/update-tracker" );
 #if GTK_CHECK_VERSION( 2, 12, 0 )
     g_signal_connect( w, "query-tooltip",
@@ -558,44 +553,29 @@ tr_window_new( GtkUIManager * ui_manager,
 #endif
 
     /* toolbar */
-    w = p->toolbar = action_get_widget( "/main-window-toolbar" );
-    gtk_box_pack_start( GTK_BOX( vbox ), w, FALSE, FALSE, 0 );
+    w = toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
 
     /* filter */
-    toggles = NULL;
-    h = p->filter = gtk_hbox_new( FALSE, 0 );
+    h = filter = p->filter = gtk_hbox_new( FALSE, 0 );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
     for( i = 0; i < FILTER_MODE_QTY; ++i )
     {
         const char * mnemonic = _( filter_names[i] );
         w = gtk_toggle_button_new_with_mnemonic( mnemonic );
-        g_object_set_data( G_OBJECT( w ), FILTER_MODE_KEY,
-                          GINT_TO_POINTER( i ) );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
-        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(
-                                          w ), i == FILTER_MODE_ALL );
-        toggles = g_slist_prepend( toggles, w );
+        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), i == FILTER_MODE_ALL );
+        p->filter_toggles[i] = GTK_TOGGLE_BUTTON( w );
         g_signal_connect( w, "toggled", G_CALLBACK( filter_toggled_cb ), p );
         gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     }
-    for( l = toggles; l != NULL; l = l->next )
-        g_object_set_data( G_OBJECT( l->data ), FILTER_TOGGLES_KEY, toggles );
     s = sexy_icon_entry_new( );
     sexy_icon_entry_add_clear_button( SEXY_ICON_ENTRY( s ) );
     image = gtk_image_new_from_stock( GTK_STOCK_FIND, GTK_ICON_SIZE_MENU );
-    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY(
-                                 s ), SEXY_ICON_ENTRY_PRIMARY,
-                             GTK_IMAGE( image ) );
-    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY(
-                                            s ), SEXY_ICON_ENTRY_PRIMARY,
-                                        TRUE );
+    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY( s ), SEXY_ICON_ENTRY_PRIMARY, GTK_IMAGE( image ) );
+    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY( s ), SEXY_ICON_ENTRY_PRIMARY, TRUE );
     gtk_box_pack_end( GTK_BOX( h ), s, FALSE, FALSE, 0 );
-    gtk_box_pack_start( GTK_BOX( vbox ), h, FALSE, FALSE, 0 );
     g_signal_connect( s, "changed", G_CALLBACK( filter_entry_changed ), p );
 
-    w = gtk_hseparator_new( );
-    gtk_box_pack_start( GTK_BOX( vbox ), w, FALSE, FALSE, 0 );
-
     /* status menu */
     menu = p->status_menu = gtk_menu_new( );
     l = NULL;
@@ -617,7 +597,7 @@ tr_window_new( GtkUIManager * ui_manager,
     }
 
     /* status */
-    h = p->status = gtk_hbox_new( FALSE, GUI_PAD );
+    h = status = p->status = gtk_hbox_new( FALSE, GUI_PAD );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD );
     w = p->gutter_lb = gtk_label_new( "N Torrents" );
     gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
@@ -644,7 +624,6 @@ tr_window_new( GtkUIManager * ui_manager,
     gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     g_signal_connect( w, "button-release-event",
                       G_CALLBACK( onYinYangReleased ), p );
-    gtk_box_pack_start( GTK_BOX( vbox ), h, FALSE, FALSE, 0 );
 
     menu = gtk_menu_new( );
     l = NULL;
@@ -653,27 +632,46 @@ tr_window_new( GtkUIManager * ui_manager,
         const char * name = _( filter_text_names[i] );
         GtkWidget *  w = gtk_radio_menu_item_new_with_label ( l, name );
         l = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM( w ) );
-        g_object_set_data( G_OBJECT(
-                              w ), FILTER_TEXT_MODE_KEY, GINT_TO_POINTER( i ) );
-        g_signal_connect( w, "toggled", G_CALLBACK(
-                              filter_text_toggled_cb ), p );
+        g_object_set_data( G_OBJECT( w ), FILTER_TEXT_MODE_KEY,
+                           GINT_TO_POINTER( i ) );
+        g_signal_connect( w, "toggled",
+                          G_CALLBACK( filter_text_toggled_cb ), p );
         gtk_menu_shell_append( GTK_MENU_SHELL( menu ), w );
         gtk_widget_show( w );
     }
-    g_signal_connect( s, "icon-released", G_CALLBACK(
-                          entry_icon_released ), menu );
+    g_signal_connect( s, "icon-released",
+                      G_CALLBACK( entry_icon_released ), menu );
 
     /* workarea */
     p->view = makeview( p, core );
-    w = p->scroll = gtk_scrolled_window_new( NULL, NULL );
-    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(
-                                        w ), GTK_POLICY_NEVER,
-                                    GTK_POLICY_AUTOMATIC );
-    gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW(
-                                             w ), GTK_SHADOW_IN );
+    w = list = p->scroll = gtk_scrolled_window_new( NULL, NULL );
+    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( w ),
+                                    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC );
+    gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( w ),
+                                         GTK_SHADOW_IN );
     gtk_container_add( GTK_CONTAINER( w ), p->view );
-    gtk_box_pack_start( GTK_BOX( vbox ), w, TRUE, TRUE, 0 );
-    gtk_container_set_focus_child( GTK_CONTAINER( vbox ), w );
+
+    /* layout the widgets */
+    {
+        const char * str = pref_string_get( PREF_KEY_MAIN_WINDOW_LAYOUT_ORDER );
+        char ** tokens = g_strsplit( str, ",", -1 );
+        for( i=0; tokens && tokens[i]; ++i )
+        {
+            const char * key = tokens[i];
+
+            if( !strcmp( key, "menu" ) )
+                gtk_box_pack_start( GTK_BOX( vbox ), mainmenu, FALSE, FALSE, 0 );
+            else if( !strcmp( key, "toolbar" ) )
+                gtk_box_pack_start( GTK_BOX( vbox ), toolbar, FALSE, FALSE, 0 );
+            else if( !strcmp( key, "filter" ) )
+                gtk_box_pack_start( GTK_BOX( vbox ), filter, FALSE, FALSE, 0 );
+            else if( !strcmp( key, "list" ) )
+                gtk_box_pack_start( GTK_BOX( vbox ), list, TRUE, TRUE, 0 );
+            else if( !strcmp( key, "statusbar" ) )
+                gtk_box_pack_start( GTK_BOX( vbox ), status, FALSE, FALSE, 0 );
+        }
+        g_strfreev( tokens );
+    }
 
     /* show all but the window */
     gtk_widget_show_all( vbox );
@@ -739,7 +737,7 @@ updateStats( PrivateData * p )
         tr_sessionGetStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: do not translate the "size|" disambiguation prefix.
+        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
@@ -750,7 +748,7 @@ updateStats( PrivateData * p )
         tr_sessionGetCumulativeStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: do not translate the "size|" disambiguation prefix.
+        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
