@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: util.c 7463 2008-12-22 05:39:02Z charles $
+ * $Id: util.c 8205 2009-04-10 17:44:17Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #include <ctype.h> /* isxdigit() */
+#include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcmp() */
@@ -121,11 +122,11 @@ tr_strlspeed( char * buf,
     if( speed < 1000.0 )  /* 0.0 KB to 999.9 KB */
         g_snprintf( buf, buflen, _( "%'.1f KB/s" ), speed );
     else if( speed < 102400.0 ) /* 0.98 MB to 99.99 MB */
-        g_snprintf( buf, buflen, _( "%'.2f MB/s" ), ( speed / 1024 ) );
+        g_snprintf( buf, buflen, _( "%'.2f MB/s" ), ( speed / KILOBYTE_FACTOR ) );
     else if( speed < 1024000.0 ) /* 100.0 MB to 999.9 MB */
-        g_snprintf( buf, buflen, _( "%'.1f MB/s" ), ( speed / 1024 ) );
+        g_snprintf( buf, buflen, _( "%'.1f MB/s" ), ( speed / MEGABYTE_FACTOR ) );
     else /* insane speeds */
-        g_snprintf( buf, buflen, _( "%'.2f GB/s" ), ( speed / 1048576 ) );
+        g_snprintf( buf, buflen, _( "%'.2f GB/s" ), ( speed / GIGABYTE_FACTOR ) );
 
     return buf;
 }
@@ -514,8 +515,8 @@ tr_object_ref_sink( gpointer object )
     return object;
 }
 
-void
-tr_file_trash_or_unlink( const char * filename )
+int
+tr_file_trash_or_remove( const char * filename )
 {
     if( filename && *filename )
     {
@@ -524,11 +525,20 @@ tr_file_trash_or_unlink( const char * filename )
         GError * err = NULL;
         GFile *  file = g_file_new_for_path( filename );
         trashed = g_file_trash( file, NULL, &err );
+        if( err )
+            g_message( "Unable to trash file \"%s\": %s", filename, err->message );
+        g_clear_error( &err );
         g_object_unref( G_OBJECT( file ) );
 #endif
-        if( !trashed )
-            g_unlink( filename );
+
+        if( !trashed && g_remove( filename ) )
+        {
+            const int err = errno;
+            g_message( "Unable to remove file \"%s\": %s", filename, g_strerror( err ) );
+        }
     }
+
+    return 0;
 }
 
 char*
@@ -559,7 +569,7 @@ gtr_open_file( const char * path )
 #endif
         if( !opened )
         {
-            char * argv[] = { "xdg-open", (char*)path, NULL };
+            char * argv[] = { (char*)"xdg-open", (char*)path, NULL };
             g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
                            NULL, NULL, NULL, NULL );
         }
@@ -646,13 +656,24 @@ gtr_button_new_from_stock( const char * stock,
 ***/
 
 guint
-gtr_timeout_add_seconds( guint        interval,
-                         GSourceFunc  function,
-                         gpointer     data )
+gtr_timeout_add_seconds( guint seconds, GSourceFunc function, gpointer data )
 {
 #if GLIB_CHECK_VERSION( 2,14,0 )
-    return g_timeout_add_seconds( interval, function, data );
+    return g_timeout_add_seconds( seconds, function, data );
 #else
-    return g_timeout_add( interval*1000, function, data );
+    return g_timeout_add( seconds*1000, function, data );
+#endif
+}
+
+void
+gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip )
+{
+#if GTK_CHECK_VERSION( 2,12,0 )
+    gtk_widget_set_tooltip_text( w, tip );
+#else
+    static GtkTooltips * tips = NULL;
+    if( tips == NULL )
+        tips = gtk_tooltips_new( );
+    gtk_tooltips_set_tip( tips, w, tip, NULL );
 #endif
 }
