@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: bencode.c 8184 2009-04-09 00:26:54Z charles $
  */
 
 #include <assert.h>
@@ -30,6 +30,10 @@
 #include "ptrarray.h"
 #include "utils.h" /* tr_new(), tr_free() */
 
+#ifndef ENODATA
+ #define ENODATA EIO
+#endif
+
 /**
 ***
 **/
@@ -379,14 +383,10 @@ tr_bencDictFind( tr_benc * val, const char * key )
     return i < 0 ? NULL : &val->val.l.vals[i + 1];
 }
 
-static tr_benc*
-tr_bencDictFindType( tr_benc *    val,
-                     const char * key,
-                     int          type )
+static tr_bool
+tr_bencDictFindType( tr_benc * dict, const char * key, int type, tr_benc ** setme )
 {
-    tr_benc * ret = tr_bencDictFind( val, key );
-
-    return ( ret && ( ret->type == type ) ) ? ret : NULL;
+    return tr_bencIsType( *setme = tr_bencDictFind( dict, key ), type );
 }
 
 size_t
@@ -432,13 +432,7 @@ tr_bencGetStr( const tr_benc * val,
 tr_bool
 tr_bencDictFindInt( tr_benc * dict, const char * key, int64_t * setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_INT );
-
-    if( child )
-        found = tr_bencGetInt( child, setme );
-
-    return found;
+    return tr_bencGetInt( tr_bencDictFind( dict, key ), setme );
 }
 
 tr_bool
@@ -456,49 +450,19 @@ tr_bencDictFindDouble( tr_benc * dict, const char * key, double * setme )
 tr_bool
 tr_bencDictFindList( tr_benc * dict, const char * key, tr_benc ** setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_LIST );
-
-    if( child )
-    {
-        if( setme != NULL )
-            *setme = child;
-        found = TRUE;
-    }
-
-    return found;
+    return tr_bencDictFindType( dict, key, TYPE_LIST, setme );
 }
 
 tr_bool
 tr_bencDictFindDict( tr_benc * dict, const char * key, tr_benc ** setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_DICT );
-
-    if( child )
-    {
-        if( setme != NULL )
-            *setme = child;
-        found = TRUE;
-    }
-
-    return found;
+    return tr_bencDictFindType( dict, key, TYPE_DICT, setme );
 }
 
 tr_bool
 tr_bencDictFindStr( tr_benc *  dict, const char *  key, const char ** setme )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
-
-    if( child )
-    {
-        if( setme )
-            *setme = child->val.s.s;
-        found = TRUE;
-    }
-
-    return found;
+    return tr_bencGetStr( tr_bencDictFind( dict, key ), setme );
 }
 
 tr_bool
@@ -507,14 +471,12 @@ tr_bencDictFindRaw( tr_benc         * dict,
                     const uint8_t  ** setme_raw,
                     size_t          * setme_len )
 {
-    tr_bool found = FALSE;
-    tr_benc * child = tr_bencDictFindType( dict, key, TYPE_STR );
+    tr_benc * child;
+    const tr_bool found = tr_bencDictFindType( dict, key, TYPE_STR, &child );
 
-    if( child )
-    {
+    if( found ) {
         *setme_raw = (uint8_t*) child->val.s.s;
         *setme_len = child->val.s.i;
-        found = TRUE;
     }
 
     return found;
@@ -1012,21 +974,21 @@ static void
 saveDictBeginFunc( const tr_benc * val UNUSED,
                    void *              evbuf )
 {
-    evbuffer_add_printf( evbuf, "d" );
+    evbuffer_add( evbuf, "d", 1 );
 }
 
 static void
 saveListBeginFunc( const tr_benc * val UNUSED,
                    void *              evbuf )
 {
-    evbuffer_add_printf( evbuf, "l" );
+    evbuffer_add( evbuf, "l", 1 );
 }
 
 static void
 saveContainerEndFunc( const tr_benc * val UNUSED,
                       void *              evbuf )
 {
-    evbuffer_add_printf( evbuf, "e" );
+    evbuffer_add( evbuf, "e", 1 );
 }
 
 char*
@@ -1114,9 +1076,12 @@ struct jsonWalk
 static void
 jsonIndent( struct jsonWalk * data )
 {
+    char buf[1024];
     const int width = tr_list_size( data->parents ) * 4;
 
-    evbuffer_add_printf( data->out, "\n%*.*s", width, width, " " );
+    buf[0] = '\n';
+    memset( buf+1, ' ', width );
+    evbuffer_add( data->out, buf, 1+width );
 }
 
 static void
@@ -1132,10 +1097,10 @@ jsonChildFunc( struct jsonWalk * data )
             {
                 const int i = parentState->childIndex++;
                 if( !( i % 2 ) )
-                    evbuffer_add_printf( data->out, ": " );
+                    evbuffer_add( data->out, ": ", 2 );
                 else
                 {
-                    evbuffer_add_printf( data->out, ", " );
+                    evbuffer_add( data->out, ", ", 2 );
                     jsonIndent( data );
                 }
                 break;
@@ -1144,7 +1109,7 @@ jsonChildFunc( struct jsonWalk * data )
             case TYPE_LIST:
             {
                 ++parentState->childIndex;
-                evbuffer_add_printf( data->out, ", " );
+                evbuffer_add( data->out, ", ", 2 );
                 jsonIndent( data );
                 break;
             }
@@ -1190,44 +1155,25 @@ jsonStringFunc( const tr_benc * val,
     struct jsonWalk *    data = vdata;
     const unsigned char *it, *end;
 
-    evbuffer_add_printf( data->out, "\"" );
+    evbuffer_add( data->out, "\"", 1 );
     for( it = (const unsigned char*)val->val.s.s, end = it + val->val.s.i;
          it != end; ++it )
     {
         switch( *it )
         {
-            case '/':
-                evbuffer_add_printf( data->out, "\\/" ); break;
-
-            case '\b':
-                evbuffer_add_printf( data->out, "\\b" ); break;
-
-            case '\f':
-                evbuffer_add_printf( data->out, "\\f" ); break;
-
-            case '\n':
-                evbuffer_add_printf( data->out, "\\n" ); break;
-
-            case '\r':
-                evbuffer_add_printf( data->out, "\\r" ); break;
-
-            case '\t':
-                evbuffer_add_printf( data->out, "\\t" ); break;
-
-            case '"':
-                evbuffer_add_printf( data->out, "\\\"" ); break;
-
-            case '\\':
-                evbuffer_add_printf( data->out, "\\\\" ); break;
+            case '/': evbuffer_add( data->out, "\\/", 2 ); break;
+            case '\b': evbuffer_add( data->out, "\\b", 2 ); break;
+            case '\f': evbuffer_add( data->out, "\\f", 2 ); break;
+            case '\n': evbuffer_add( data->out, "\\n", 2 ); break;
+            case '\r': evbuffer_add( data->out, "\\r", 2 ); break;
+            case '\t': evbuffer_add( data->out, "\\t", 2 ); break;
+            case '"': evbuffer_add( data->out, "\\\"", 2 ); break;
+            case '\\': evbuffer_add( data->out, "\\\\", 2 ); break;
 
             default:
                 if( isascii( *it ) )
-                {
-                    /*fprintf( stderr, "[%c]\n", *it );*/
-                    evbuffer_add_printf( data->out, "%c", *it );
-                }
-                else
-                {
+                    evbuffer_add( data->out, it, 1 );
+                else {
                     const UTF8 * tmp = it;
                     UTF32        buf = 0;
                     UTF32 *      u32 = &buf;
@@ -1238,11 +1184,10 @@ jsonStringFunc( const tr_benc * val,
                         evbuffer_add_printf( data->out, "\\u%04x", (unsigned int)buf );
                         it = tmp - 1;
                     }
-                    /*fprintf( stderr, "[\\u%04x]\n", buf );*/
                 }
         }
     }
-    evbuffer_add_printf( data->out, "\"" );
+    evbuffer_add( data->out, "\"", 1 );
     jsonChildFunc( data );
 }
 
@@ -1253,7 +1198,7 @@ jsonDictBeginFunc( const tr_benc * val,
     struct jsonWalk * data = vdata;
 
     jsonPushParent( data, val );
-    evbuffer_add_printf( data->out, "{" );
+    evbuffer_add( data->out, "{", 1 );
     if( val->val.l.count )
         jsonIndent( data );
 }
@@ -1266,7 +1211,7 @@ jsonListBeginFunc( const tr_benc * val,
     struct jsonWalk * data = vdata;
 
     jsonPushParent( data, val );
-    evbuffer_add_printf( data->out, "[" );
+    evbuffer_add( data->out, "[", 1 );
     if( nChildren )
         jsonIndent( data );
 }
@@ -1296,9 +1241,9 @@ jsonContainerEndFunc( const tr_benc * val,
     if( !emptyContainer )
         jsonIndent( data );
     if( tr_bencIsDict( val ) )
-        evbuffer_add_printf( data->out, "}" );
+        evbuffer_add( data->out, "}", 1 );
     else /* list */
-        evbuffer_add_printf( data->out, "]" );
+        evbuffer_add( data->out, "]", 1 );
     jsonChildFunc( data );
 }
 
@@ -1465,16 +1410,17 @@ tr_bencSaveJSONFile( const char *    filename,
 ***/
 
 int
-tr_bencLoadFile( const char * filename,
-                 tr_benc *    b )
+tr_bencLoadFile( const char * filename, tr_benc * b )
 {
     int       err;
     size_t    contentLen;
     uint8_t * content;
 
     content = tr_loadFile( filename, &contentLen );
-    if( !content )
+    if( !content && errno )
         err = errno;
+    else if( !content )
+        err = ENODATA;
     else
         err = tr_bencLoad( content, contentLen, b, NULL );
 
@@ -1483,20 +1429,20 @@ tr_bencLoadFile( const char * filename,
 }
 
 int
-tr_bencLoadJSONFile( const char * filename,
-                     tr_benc *    b )
+tr_bencLoadJSONFile( const char * filename, tr_benc * b )
 {
     int        err;
     size_t     contentLen;
     uint8_t  * content;
 
     content = tr_loadFile( filename, &contentLen );
-    if( !content )
+    if( !content && errno )
         err = errno;
+    else if( !content )
+        err = ENODATA;
     else
         err = tr_jsonParse( content, contentLen, b, NULL );
 
     tr_free( content );
     return err;
 }
-
