@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.c 7142 2008-11-23 17:27:39Z charles $
+ * $Id: fdlimit.c 8203 2009-04-10 15:56:40Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -26,6 +26,13 @@
  #define HAVE_GETRLIMIT
 #endif
 
+#ifdef HAVE_POSIX_FADVISE
+ #ifdef _XOPEN_SOURCE
+  #undef _XOPEN_SOURCE
+ #endif
+ #define _XOPEN_SOURCE 600
+#endif
+
 #include <assert.h>
 #include <errno.h>
 #include <inttypes.h>
@@ -33,7 +40,7 @@
 #include <stdlib.h>
 #include <string.h>
 #ifdef SYS_DARWIN
-#include <fcntl.h>
+ #include <fcntl.h>
 #endif
 
 #ifdef HAVE_FALLOCATE
@@ -47,9 +54,8 @@
  #include <sys/resource.h> /* getrlimit */
 #endif
 #include <unistd.h>
-#include <fcntl.h> /* O_LARGEFILE */
+#include <fcntl.h> /* O_LARGEFILE posix_fadvise */
 
-#include <event.h>
 #include <evutil.h>
 
 #include "transmission.h"
@@ -71,28 +77,33 @@
 
 enum
 {
-    TR_MAX_OPEN_FILES = 16, /* real files, not sockets */
-
     NOFILE_BUFFER = 512, /* the process' number of open files is
                             globalMaxPeers + NOFILE_BUFFER */
+
+    SYNC_INTERVAL = 15   /* (arbitrary number) how many seconds to go
+                            between fsync calls for files in heavy use */
 };
 
 struct tr_openfile
 {
-    unsigned int    isCheckedOut  : 1;
-    unsigned int    isWritable    : 1;
-    unsigned int    closeWhenDone : 1;
-    char            filename[MAX_PATH_LENGTH];
-    int             fd;
-    uint64_t        date;
+    tr_bool    isCheckedOut;
+    tr_bool    isWritable;
+    tr_bool    closeWhenDone;
+    char       filename[MAX_PATH_LENGTH];
+    int        fd;
+    uint64_t   date;
+    time_t     syncAt;
 };
 
 struct tr_fd_s
 {
     int                   socketCount;
-    int                   socketMax;
-    tr_lock *             lock;
-    struct tr_openfile    open[TR_MAX_OPEN_FILES];
+    int                   socketLimit;
+
+    struct tr_openfile  * openFiles;
+    int                   openFileLimit;
+
+    tr_lock             * lock;
 };
 
 static struct tr_fd_s * gFd = NULL;
@@ -104,13 +115,31 @@ static struct tr_fd_s * gFd = NULL;
 ***/
 
 #ifndef O_LARGEFILE
-#define O_LARGEFILE 0
+ #define O_LARGEFILE 0
 #endif
 
-static int
-preallocateFile( const char * filename, uint64_t length )
+static tr_bool
+preallocateFileSparse( int fd, uint64_t length )
 {
-    int success = 0;
+    const char zero = '\0';
+
+    if( length == 0 )
+        return TRUE;
+
+    if( lseek( fd, length-1, SEEK_SET ) == -1 )
+        return FALSE;
+    if( write( fd, &zero, 1 ) == -1 )
+        return FALSE;
+    if( ftruncate( fd, length ) == -1 )
+        return FALSE;
+
+    return TRUE;
+}
+
+static tr_bool
+preallocateFileFull( const char * filename, uint64_t length )
+{
+    tr_bool success = 0;
 
 #ifdef WIN32
 
@@ -169,14 +198,14 @@ preallocateFile( const char * filename, uint64_t length )
  * plus the errno values set by tr_mkdirp() and open().
  */
 static int
-TrOpenFile( int          i,
-            const char * folder,
-            const char * torrentFile,
-            int          doWrite,
-            int          doPreallocate,
-            uint64_t     desiredFileSize )
+TrOpenFile( int                      i,
+            const char             * folder,
+            const char             * torrentFile,
+            tr_bool                  doWrite,
+            tr_preallocation_mode    preallocationMode,
+            uint64_t                 desiredFileSize )
 {
-    struct tr_openfile * file = &gFd->open[i];
+    struct tr_openfile * file = &gFd->openFiles[i];
     int                  flags;
     char               * filename;
     struct stat          sb;
@@ -184,7 +213,10 @@ TrOpenFile( int          i,
 
     /* confirm the parent folder exists */
     if( stat( folder, &sb ) || !S_ISDIR( sb.st_mode ) )
+    {
+        tr_err( _( "Couldn't create \"%1$s\": parent folder \"%2$s\" does not exist" ), torrentFile, folder );
         return ENOENT;
+    }
 
     /* create subfolders, if any */
     filename = tr_buildPath( folder, torrentFile, NULL );
@@ -192,21 +224,26 @@ TrOpenFile( int          i,
     {
         char * tmp = tr_dirname( filename );
         const int err = tr_mkdirp( tmp, 0777 ) ? errno : 0;
-        tr_free( tmp );
         if( err ) {
+            tr_err( _( "Couldn't create \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
+            tr_free( tmp );
             tr_free( filename );
             return err;
         }
+        tr_free( tmp );
     }
 
     alreadyExisted = !stat( filename, &sb ) && S_ISREG( sb.st_mode );
 
-    if( doWrite && !alreadyExisted && doPreallocate )
-        if( preallocateFile( filename, desiredFileSize ) )
+    if( doWrite && !alreadyExisted && ( preallocationMode == TR_PREALLOCATE_FULL ) )
+        if( preallocateFileFull( filename, desiredFileSize ) )
             tr_inf( _( "Preallocated file \"%s\"" ), filename );
     
     /* open the file */
     flags = doWrite ? ( O_RDWR | O_CREAT ) : O_RDONLY;
+#ifdef O_RANDOM
+    flags |= O_RANDOM
+#endif
 #ifdef O_LARGEFILE
     flags |= O_LARGEFILE;
 #endif
@@ -217,12 +254,18 @@ TrOpenFile( int          i,
     if( file->fd == -1 )
     {
         const int err = errno;
-        tr_err( _( "Couldn't open \"%1$s\": %2$s" ), filename,
-               tr_strerror( err ) );
+        tr_err( _( "Couldn't open \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
         tr_free( filename );
         return err;
     }
 
+    if( doWrite && !alreadyExisted && ( preallocationMode == TR_PREALLOCATE_SPARSE ) )
+        preallocateFileSparse( file->fd, desiredFileSize );
+
+#ifdef HAVE_POSIX_FADVISE
+    posix_fadvise( file->fd, 0, 0, POSIX_FADV_RANDOM );
+#endif
+
     tr_free( filename );
     return 0;
 }
@@ -236,10 +279,10 @@ fileIsOpen( const struct tr_openfile * o )
 static void
 TrCloseFile( int i )
 {
-    struct tr_openfile * o = &gFd->open[i];
+    struct tr_openfile * o = &gFd->openFiles[i];
 
     assert( i >= 0 );
-    assert( i < TR_MAX_OPEN_FILES );
+    assert( i < gFd->openFileLimit );
     assert( fileIsOpen( o ) );
 
     close( o->fd );
@@ -255,30 +298,29 @@ fileIsCheckedOut( const struct tr_openfile * o )
 
 /* returns an fd on success, or a -1 on failure and sets errno */
 int
-tr_fdFileCheckout( const char * folder,
-                   const char * torrentFile,
-                   int          doWrite,
-                   int          doPreallocate,
-                   uint64_t     desiredFileSize )
+tr_fdFileCheckout( const char             * folder,
+                   const char             * torrentFile,
+                   tr_bool                  doWrite,
+                   tr_preallocation_mode    preallocationMode,
+                   uint64_t                 desiredFileSize )
 {
-    int                  i, winner = -1;
+    int i, winner = -1;
     struct tr_openfile * o;
-    char               * filename;
+    char filename[MAX_PATH_LENGTH];
 
     assert( folder && *folder );
     assert( torrentFile && *torrentFile );
     assert( doWrite == 0 || doWrite == 1 );
 
-    filename = tr_buildPath( folder, torrentFile, NULL );
-    dbgmsg( "looking for file '%s', writable %c", filename,
-            doWrite ? 'y' : 'n' );
+    tr_snprintf( filename, sizeof( filename ), "%s%c%s", folder, TR_PATH_DELIMITER, torrentFile );
+    dbgmsg( "looking for file '%s', writable %c", filename, doWrite ? 'y' : 'n' );
 
     tr_lockLock( gFd->lock );
 
     /* Is it already open? */
-    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < gFd->openFileLimit; ++i )
     {
-        o = &gFd->open[i];
+        o = &gFd->openFiles[i];
 
         if( !fileIsOpen( o ) )
             continue;
@@ -316,9 +358,9 @@ tr_fdFileCheckout( const char * folder,
         uint64_t date = tr_date( ) + 1;
 
         /* look for the file that's been open longest */
-        for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
+        for( i = 0; i < gFd->openFileLimit; ++i )
         {
-            o = &gFd->open[i];
+            o = &gFd->openFiles[i];
 
             if( !fileIsOpen( o ) )
             {
@@ -336,18 +378,17 @@ tr_fdFileCheckout( const char * folder,
 
         if( winner >= 0 )
         {
-            if( fileIsOpen( &gFd->open[winner] ) )
+            if( fileIsOpen( &gFd->openFiles[winner] ) )
             {
                 dbgmsg( "closing file '%s', slot #%d",
-                        gFd->open[winner].filename,
+                        gFd->openFiles[winner].filename,
                         winner );
                 TrCloseFile( winner );
             }
         }
         else
         {
-            dbgmsg(
-                "everything's full!  waiting for someone else to finish something" );
+            dbgmsg( "everything's full!  waiting for someone else to finish something" );
             tr_lockUnlock( gFd->lock );
             tr_wait( 200 );
             tr_lockLock( gFd->lock );
@@ -355,13 +396,12 @@ tr_fdFileCheckout( const char * folder,
     }
 
     assert( winner >= 0 );
-    o = &gFd->open[winner];
+    o = &gFd->openFiles[winner];
     if( !fileIsOpen( o ) )
     {
-        const int err = TrOpenFile( winner, folder, torrentFile, doWrite, doPreallocate, desiredFileSize );
+        const int err = TrOpenFile( winner, folder, torrentFile, doWrite, preallocationMode, desiredFileSize );
         if( err ) {
             tr_lockUnlock( gFd->lock );
-            tr_free( filename );
             errno = err;
             return -1;
         }
@@ -370,13 +410,13 @@ tr_fdFileCheckout( const char * folder,
                 doWrite ? 'y' : 'n' );
         tr_strlcpy( o->filename, filename, sizeof( o->filename ) );
         o->isWritable = doWrite;
+        o->syncAt = time( NULL ) + SYNC_INTERVAL;
     }
 
     dbgmsg( "checking out '%s' in slot %d", filename, winner );
     o->isCheckedOut = 1;
     o->closeWhenDone = 0;
     o->date = tr_date( );
-    tr_free( filename );
     tr_lockUnlock( gFd->lock );
     return o->fd;
 }
@@ -388,9 +428,9 @@ tr_fdFileReturn( int fd )
 
     tr_lockLock( gFd->lock );
 
-    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < gFd->openFileLimit; ++i )
     {
-        struct tr_openfile * o = &gFd->open[i];
+        struct tr_openfile * o = &gFd->openFiles[i];
         if( o->fd != fd )
             continue;
 
@@ -398,6 +438,15 @@ tr_fdFileReturn( int fd )
         o->isCheckedOut = 0;
         if( o->closeWhenDone )
             TrCloseFile( i );
+        else if( o->syncAt <= time( NULL ) ) {
+            dbgmsg( "fsync()ing file '%s' in slot #%d", o->filename, i );
+            fsync( o->fd );
+#ifdef HAVE_POSIX_FADVISE
+            /* TODO: test performance with and without this */
+            posix_fadvise( o->fd, 0, 0, POSIX_FADV_DONTNEED );
+#endif
+            o->syncAt = time( NULL ) + SYNC_INTERVAL;
+        }
 
         break;
     }
@@ -412,9 +461,9 @@ tr_fdFileClose( const char * filename )
 
     tr_lockLock( gFd->lock );
 
-    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
+    for( i = 0; i < gFd->openFileLimit; ++i )
     {
-        struct tr_openfile * o = &gFd->open[i];
+        struct tr_openfile * o = &gFd->openFiles[i];
         if( !fileIsOpen( o ) || strcmp( filename, o->filename ) )
             continue;
 
@@ -429,7 +478,7 @@ tr_fdFileClose( const char * filename )
         {
             dbgmsg(
                 "flagging file '%s', slot #%d to be closed when checked in",
-                gFd->open[i].filename, i );
+                gFd->openFiles[i].filename, i );
             o->closeWhenDone = 1;
         }
     }
@@ -446,20 +495,26 @@ tr_fdFileClose( const char * filename )
 static int
 getSocketMax( struct tr_fd_s * gFd )
 {
-    return gFd->socketMax;
+    return gFd->socketLimit;
 }
 
 int
-tr_fdSocketCreate( int type )
+tr_fdSocketCreate( int domain, int type )
 {
     int s = -1;
 
     tr_lockLock( gFd->lock );
 
     if( gFd->socketCount < getSocketMax( gFd ) )
-        if( ( s = socket( AF_INET, type, 0 ) ) < 0 )
+        if( ( s = socket( domain, type, 0 ) ) < 0 )
+        {
+#ifdef SYS_DARWIN
+            if( sockerrno != EAFNOSUPPORT )
+#endif
             tr_err( _( "Couldn't create socket: %s" ),
                    tr_strerror( sockerrno ) );
+            s = -sockerrno;
+        }
 
     if( s > -1 )
         ++gFd->socketCount;
@@ -471,13 +526,13 @@ tr_fdSocketCreate( int type )
 }
 
 int
-tr_fdSocketAccept( int              b,
-                   struct in_addr * addr,
-                   tr_port_t *      port )
+tr_fdSocketAccept( int           b,
+                   tr_address  * addr,
+                   tr_port     * port )
 {
     int                s = -1;
     unsigned int       len;
-    struct sockaddr_in sock;
+    struct sockaddr_storage sock;
 
     assert( addr );
     assert( port );
@@ -485,13 +540,27 @@ tr_fdSocketAccept( int              b,
     tr_lockLock( gFd->lock );
     if( gFd->socketCount < getSocketMax( gFd ) )
     {
-        len = sizeof( sock );
+        len = sizeof( struct sockaddr_storage );
         s = accept( b, (struct sockaddr *) &sock, &len );
     }
     if( s > -1 )
     {
-        *addr = sock.sin_addr;
-        *port = sock.sin_port;
+        /* "The ss_family field of the sockaddr_storage structure will always 
+         * align with the family field of any protocol-specific structure." */ 
+        if( sock.ss_family == AF_INET ) 
+        { 
+            struct sockaddr_in * sock4 = (struct sockaddr_in *)&sock; 
+            addr->type = TR_AF_INET; 
+            addr->addr.addr4.s_addr = sock4->sin_addr.s_addr; 
+            *port = sock4->sin_port; 
+        } 
+        else 
+        { 
+            struct sockaddr_in6 * sock6 = (struct sockaddr_in6 *)&sock; 
+            addr->type = TR_AF_INET6; 
+            addr->addr.addr6 = sock6->sin6_addr;
+            *port = sock6->sin6_port; 
+        } 
         ++gFd->socketCount;
     }
     tr_lockUnlock( gFd->lock );
@@ -502,11 +571,7 @@ tr_fdSocketAccept( int              b,
 static void
 socketClose( int fd )
 {
-#ifdef BEOS_NETSERVER
-    closesocket( fd );
-#else
     EVUTIL_CLOSESOCKET( fd );
-#endif
 }
 
 void
@@ -532,12 +597,14 @@ tr_fdSocketClose( int s )
 ***/
 
 void
-tr_fdInit( int globalPeerLimit )
+tr_fdInit( size_t openFileLimit, size_t socketLimit )
 {
     int i;
 
     assert( gFd == NULL );
     gFd = tr_new0( struct tr_fd_s, 1 );
+    gFd->openFiles = tr_new0( struct tr_openfile, openFileLimit );
+    gFd->openFileLimit = openFileLimit;
     gFd->lock = tr_lockNew( );
 
 #ifdef HAVE_GETRLIMIT
@@ -545,18 +612,18 @@ tr_fdInit( int globalPeerLimit )
         struct rlimit rlim;
         getrlimit( RLIMIT_NOFILE, &rlim );
         rlim.rlim_cur = MIN( rlim.rlim_max,
-                            (rlim_t)( globalPeerLimit + NOFILE_BUFFER ) );
+                            (rlim_t)( socketLimit + NOFILE_BUFFER ) );
         setrlimit( RLIMIT_NOFILE, &rlim );
-        gFd->socketMax = rlim.rlim_cur - NOFILE_BUFFER;
+        gFd->socketLimit = rlim.rlim_cur - NOFILE_BUFFER;
         tr_dbg( "setrlimit( RLIMIT_NOFILE, %d )", (int)rlim.rlim_cur );
     }
 #else
-    gFd->socketMax = globalPeerLimit;
+    gFd->socketLimit = socketLimit;
 #endif
-    tr_dbg( "%d usable file descriptors", globalPeerLimit );
+    tr_dbg( "%zu usable file descriptors", socketLimit );
 
-    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
-        gFd->open[i].fd = -1;
+    for( i = 0; i < gFd->openFileLimit; ++i )
+        gFd->openFiles[i].fd = -1;
 }
 
 void
@@ -564,12 +631,13 @@ tr_fdClose( void )
 {
     int i = 0;
 
-    for( i = 0; i < TR_MAX_OPEN_FILES; ++i )
-        if( fileIsOpen( &gFd->open[i] ) )
+    for( i = 0; i < gFd->openFileLimit; ++i )
+        if( fileIsOpen( &gFd->openFiles[i] ) )
             TrCloseFile( i );
 
     tr_lockFree( gFd->lock );
 
+    tr_free( gFd->openFiles );
     tr_free( gFd );
     gFd = NULL;
 }
@@ -578,12 +646,12 @@ void
 tr_fdSetPeerLimit( uint16_t n )
 {
     assert( gFd != NULL && "tr_fdInit() must be called first!" );
-    gFd->socketMax = n;
+    gFd->socketLimit = n;
 }
 
 uint16_t
 tr_fdGetPeerLimit( void )
 {
-    return gFd ? gFd->socketMax : -1;
+    return gFd ? gFd->socketLimit : -1;
 }
 
