@@ -144,7 +144,10 @@ test2( void )
 {
     tr_benc top;
     const char * in = " ";
-    const int err = tr_jsonParse( in, strlen( in ), &top, NULL );
+    int err;
+
+    top.type = 0;
+    err = tr_jsonParse( in, strlen( in ), &top, NULL );
 
     check( err );
     check( !tr_bencIsDict( &top ) );
