@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.c 6945 2008-10-22 17:14:50Z charles $
+ * $Id: makemeta.c 7664 2009-01-11 17:46:51Z charles $
  */
 
 #include <assert.h>
@@ -23,6 +23,7 @@
 
 #include "crypto.h" /* tr_sha1 */
 #include "transmission.h"
+#include "session.h"
 #include "bencode.h"
 #include "makemeta.h"
 #include "platform.h" /* threads, locks */
@@ -113,7 +114,7 @@ builderFileCompare( const void * va,
 }
 
 tr_metainfo_builder*
-tr_metaInfoBuilderCreate( tr_handle *  handle,
+tr_metaInfoBuilderCreate( tr_session * session,
                           const char * topFile )
 {
     int                   i;
@@ -122,7 +123,7 @@ tr_metaInfoBuilderCreate( tr_handle *  handle,
     tr_metainfo_builder * ret = tr_new0( tr_metainfo_builder, 1 );
 
     ret->top = tr_strdup( topFile );
-    ret->handle = handle;
+    ret->handle = session;
     {
         struct stat sb;
         stat( topFile, &sb );
@@ -344,9 +345,9 @@ makeInfoDict( tr_benc *             dict,
                                              builder->fileCount );
         for( i = 0; i < builder->fileCount; ++i )
         {
-            tr_benc * dict = tr_bencListAddDict( list, 2 );
-            tr_benc * length = tr_bencDictAdd( dict, "length" );
-            tr_benc * pathVal = tr_bencDictAdd( dict, "path" );
+            tr_benc * d = tr_bencListAddDict( list, 2 );
+            tr_benc * length = tr_bencDictAdd( d, "length" );
+            tr_benc * pathVal = tr_bencDictAdd( d, "path" );
             getFileInfo( builder->top, &builder->files[i], length, pathVal );
         }
     }
@@ -444,29 +445,29 @@ static tr_metainfo_builder * queue = NULL;
 static tr_thread *           workerThread = NULL;
 
 static tr_lock*
-getQueueLock( tr_handle * h )
+getQueueLock( tr_session * session )
 {
     static tr_lock * lock = NULL;
+    tr_globalLock( session );
 
-    tr_globalLock( h );
     if( !lock )
         lock = tr_lockNew( );
-    tr_globalUnlock( h );
 
+    tr_globalUnlock( session );
     return lock;
 }
 
 static void
 makeMetaWorkerFunc( void * user_data )
 {
-    tr_handle * handle = (tr_handle *) user_data;
+    tr_session * session = user_data;
 
     for( ; ; )
     {
         tr_metainfo_builder * builder = NULL;
 
         /* find the next builder to process */
-        tr_lock *             lock = getQueueLock ( handle );
+        tr_lock * lock = getQueueLock( session );
         tr_lockLock( lock );
         if( queue )
         {
