@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: net.h 7847 2009-02-08 00:29:20Z jhujhiti $
+ * $Id: net.h 8204 2009-04-10 17:34:25Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -86,12 +86,9 @@ tr_address *tr_pton( const char * src,
                      tr_address * dst );
 int tr_compareAddresses( const tr_address * a,
                          const tr_address * b);
-void tr_normalizeV4Mapped( tr_address * const addr );
 
 tr_bool tr_isValidPeerAddress( const tr_address * addr, tr_port port );
 
-void tr_suspectAddress( const tr_address * a, const char * source );
-
 static TR_INLINE tr_bool tr_isAddress( const tr_address * a ) { return ( a != NULL ) && ( a->type==TR_AF_INET || a->type==TR_AF_INET6 ); }
 
 tr_bool tr_net_hasIPv6( tr_port );
