@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 7813 2009-01-29 18:18:24Z charles $
+ * $Id: peer-io.c 8218 2009-04-12 15:01:29Z charles $
  */
 
 #include <assert.h>
@@ -740,8 +740,8 @@ tr_peerIoReadBytes( tr_peerIo       * io,
             break;
 
         case PEER_ENCRYPTION_RC4:
-            evbuffer_remove( inbuf, bytes, byteCount );
-            tr_cryptoDecrypt( io->crypto, byteCount, bytes, bytes );
+            tr_cryptoDecrypt( io->crypto, byteCount, EVBUFFER_DATA(inbuf), bytes );
+            evbuffer_drain(inbuf, byteCount );
             break;
 
         default:
