@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpc-server.c 7767 2009-01-20 18:31:37Z charles $
+ * $Id: rpc-server.c 8204 2009-04-10 17:34:25Z charles $
  */
 
 #include <assert.h>
@@ -313,7 +313,7 @@ serve_file( struct evhttp_request * req,
 
         if( errno )
         {
-            send_simple_response( req, HTTP_NOTFOUND, NULL );
+            send_simple_response( req, HTTP_NOTFOUND, filename );
         }
         else
         {
@@ -356,20 +356,25 @@ handle_clutch( struct evhttp_request * req,
     {
         char * pch;
         char * subpath;
-        char * filename;
 
         subpath = tr_strdup( req->uri + 18 );
         if(( pch = strchr( subpath, '?' )))
             *pch = '\0';
 
-        filename = tr_strdup_printf( "%s%s%s",
-                       clutchDir,
-                       TR_PATH_DELIMITER_STR,
-                       subpath && *subpath ? subpath : "index.html" );
+        if( strstr( subpath, ".." ) )
+        {
+            send_simple_response( req, HTTP_NOTFOUND, "<p>Tsk, tsk.</p>" );
+        }
+        else
+        {
+            char * filename = tr_strdup_printf( "%s%s%s",
+                                 clutchDir,
+                                 TR_PATH_DELIMITER_STR,
+                                 subpath && *subpath ? subpath : "index.html" );
+            serve_file( req, server, filename );
+            tr_free( filename );
+        }
 
-        serve_file( req, server, filename );
-
-        tr_free( filename );
         tr_free( subpath );
     }
 }
@@ -488,9 +493,13 @@ handle_request( struct evhttp_request * req,
                || !strcmp( req->uri, "/transmission/clutch" )
                || !strcmp( req->uri, "/" ) )
         {
-            evhttp_add_header( req->output_headers, "Location",
-                               "/transmission/web/" );
+            const char * protocol = "http";
+            const char * host = evhttp_find_header( req->input_headers, "Host" );
+            const char * uri = "transmission/web/";
+            char * location = tr_strdup_printf( "%s://%s/%s", protocol, host, uri );
+            evhttp_add_header( req->output_headers, "Location", location );
             send_simple_response( req, HTTP_MOVEPERM, NULL );
+            tr_free( location );
         }
         else if( !strncmp( req->uri, "/transmission/web/", 18 ) )
         {
@@ -506,7 +515,7 @@ handle_request( struct evhttp_request * req,
         }
         else
         {
-            send_simple_response( req, HTTP_NOTFOUND, NULL );
+            send_simple_response( req, HTTP_NOTFOUND, req->uri );
         }
 
         tr_free( user );
