@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpcimpl.c 7349 2008-12-10 21:48:46Z charles $
+ * $Id: rpcimpl.c 8005 2009-03-03 20:06:44Z charles $
  */
 
 #include <assert.h>
@@ -15,22 +15,41 @@
 #include <stdlib.h> /* strtol */
 #include <string.h> /* strcmp */
 
+#include <event.h> /* evbuffer */
+
 #include "transmission.h"
 #include "bencode.h"
 #include "rpcimpl.h"
 #include "json.h"
 #include "session.h"
+#include "stats.h"
 #include "torrent.h"
 #include "utils.h"
+#include "web.h"
 
 #define TR_N_ELEMENTS( ary ) ( sizeof( ary ) / sizeof( *ary ) )
 
+#if 0
+#define dbgmsg(fmt, ...) \
+    do { \
+        fprintf( stderr, "%s:%d"#fmt, __FILE__, __LINE__, __VA_ARGS__ ); \
+        fprintf( stderr, "\n" ); \
+    } while( 0 )
+#else
+#define dbgmsg( ... ) \
+    do { \
+        if( tr_deepLoggingIsActive( ) ) \
+            tr_deepLog( __FILE__, __LINE__, "RPC", __VA_ARGS__ ); \
+    } while( 0 )
+#endif
+
+
 /***
 ****
 ***/
 
 static tr_rpc_callback_status
-notify( tr_handle *  session,
+notify( tr_session * session,
         int          type,
         tr_torrent * tor )
 {
@@ -47,10 +66,45 @@ notify( tr_handle *  session,
 ****
 ***/
 
+/* For functions that can't be immediately executed, like torrentAdd,
+ * this is the callback data used to pass a response to the caller
+ * when the task is complete */
+struct tr_rpc_idle_data
+{
+    tr_session            * session;
+    tr_benc               * response;
+    tr_benc               * args_out;
+    tr_rpc_response_func    callback;
+    void                  * callback_user_data;
+};
+
+static void
+tr_idle_function_done( struct tr_rpc_idle_data * data, const char * result )
+{
+    struct evbuffer * buf = tr_getBuffer( );
+
+    if( result == NULL )
+        result = "success";
+    tr_bencDictAddStr( data->response, "result", result );
+
+    tr_bencSaveAsJSON( data->response, buf );
+    (*data->callback)( data->session, (const char*)EVBUFFER_DATA(buf),
+                       EVBUFFER_LENGTH(buf), data->callback_user_data );
+
+    tr_releaseBuffer( buf );
+    tr_bencFree( data->response );
+    tr_free( data->response );
+    tr_free( data );
+}
+
+/***
+****
+***/
+
 static tr_torrent **
-getTorrents( tr_handle * handle,
-             tr_benc *   args,
-             int *       setmeCount )
+getTorrents( tr_session * session,
+             tr_benc    * args,
+             int        * setmeCount )
 {
     int           torrentCount = 0;
     int64_t       id;
@@ -68,12 +122,11 @@ getTorrents( tr_handle * handle,
         {
             tr_torrent * tor = NULL;
             tr_benc *    node = tr_bencListChild( ids, i );
-            int64_t      id;
             const char * str;
             if( tr_bencGetInt( node, &id ) )
-                tor = tr_torrentFindFromId( handle, id );
+                tor = tr_torrentFindFromId( session, id );
             else if( tr_bencGetStr( node, &str ) )
-                tor = tr_torrentFindFromHashString( handle, str );
+                tor = tr_torrentFindFromHashString( session, str );
             if( tor )
                 torrents[torrentCount++] = tor;
         }
@@ -83,15 +136,15 @@ getTorrents( tr_handle * handle,
     {
         tr_torrent * tor;
         torrents = tr_new0( tr_torrent *, 1 );
-        if( ( tor = tr_torrentFindFromId( handle, id ) ) )
+        if( ( tor = tr_torrentFindFromId( session, id ) ) )
             torrents[torrentCount++] = tor;
     }
     else /* all of them */
     {
         tr_torrent * tor = NULL;
-        const int    n = tr_sessionCountTorrents( handle );
+        const int    n = tr_sessionCountTorrents( session );
         torrents = tr_new0( tr_torrent *, n );
-        while( ( tor = tr_torrentNext( handle, tor ) ) )
+        while( ( tor = tr_torrentNext( session, tor ) ) )
             torrents[torrentCount++] = tor;
     }
 
@@ -100,75 +153,92 @@ getTorrents( tr_handle * handle,
 }
 
 static const char*
-torrentStart( tr_handle *        h,
-              tr_benc *          args_in,
-              tr_benc * args_out UNUSED )
+torrentStart( tr_session               * session,
+              tr_benc                  * args_in,
+              tr_benc                  * args_out UNUSED,
+              struct tr_rpc_idle_data  * idle_data )
 {
     int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( h, args_in, &torrentCount );
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+
+    assert( idle_data == NULL );
 
     for( i = 0; i < torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
         tr_torrentStart( tor );
-        notify( h, TR_RPC_TORRENT_STARTED, tor );
+        notify( session, TR_RPC_TORRENT_STARTED, tor );
     }
     tr_free( torrents );
     return NULL;
 }
 
 static const char*
-torrentStop( tr_handle *        h,
-             tr_benc *          args_in,
-             tr_benc * args_out UNUSED )
+torrentStop( tr_session               * session,
+             tr_benc                  * args_in,
+             tr_benc                  * args_out UNUSED,
+             struct tr_rpc_idle_data  * idle_data )
 {
     int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( h, args_in, &torrentCount );
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+
+    assert( idle_data == NULL );
 
     for( i = 0; i < torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
         tr_torrentStop( tor );
-        notify( h, TR_RPC_TORRENT_STOPPED, tor );
+        notify( session, TR_RPC_TORRENT_STOPPED, tor );
     }
     tr_free( torrents );
     return NULL;
 }
 
 static const char*
-torrentRemove( tr_handle *        h,
-               tr_benc *          args_in,
-               tr_benc * args_out UNUSED )
+torrentRemove( tr_session               * session,
+               tr_benc                  * args_in,
+               tr_benc                  * args_out UNUSED,
+               struct tr_rpc_idle_data  * idle_data )
 {
-    int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( h, args_in, &torrentCount );
+    int i;
+    int torrentCount;
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
 
-    for( i = 0; i < torrentCount; ++i )
+    assert( idle_data == NULL );
+
+    for( i=0; i<torrentCount; ++i )
     {
-        tr_torrent *                 tor = torrents[i];
-        const tr_rpc_callback_status status = notify(
-            h, TR_RPC_TORRENT_REMOVING, tor );
+        tr_torrent * tor = torrents[i];
+        const tr_rpc_callback_status status = notify( session, TR_RPC_TORRENT_REMOVING, tor );
+        int64_t deleteFlag;
+        if( tr_bencDictFindInt( args_in, "delete-local-data", &deleteFlag ) && deleteFlag )
+            tr_torrentDeleteLocalData( tor, NULL );
         if( !( status & TR_RPC_NOREMOVE ) )
             tr_torrentRemove( tor );
     }
+
     tr_free( torrents );
     return NULL;
 }
 
 static const char*
-torrentVerify( tr_handle *        h,
-               tr_benc *          args_in,
-               tr_benc * args_out UNUSED )
+torrentVerify( tr_session               * session,
+               tr_benc                  * args_in,
+               tr_benc                  * args_out UNUSED,
+               struct tr_rpc_idle_data  * idle_data )
 {
     int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( h, args_in, &torrentCount );
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+
+    assert( idle_data == NULL );
 
     for( i = 0; i < torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
         tr_torrentVerify( tor );
-        notify( h, TR_RPC_TORRENT_CHANGED, tor );
+        notify( session, TR_RPC_TORRENT_CHANGED, tor );
     }
+
     tr_free( torrents );
     return NULL;
 }
@@ -289,8 +359,12 @@ addField( const tr_torrent * tor,
         tr_bencDictAddInt( d, key, st->desiredAvailable );
     else if( !strcmp( key, "doneDate" ) )
         tr_bencDictAddInt( d, key, st->doneDate );
+    else if( !strcmp( key, "downloadDir" ) )
+        tr_bencDictAddStr( d, key, tr_torrentGetDownloadDir( tor ) );
     else if( !strcmp( key, "downloadedEver" ) )
         tr_bencDictAddInt( d, key, st->downloadedEver );
+    else if( !strcmp( key, "downloaders" ) )
+        tr_bencDictAddInt( d, key, st->downloaders );
     else if( !strcmp( key, "downloadLimitMode" ) )
         tr_bencDictAddInt( d, key, tr_torrentGetSpeedMode( tor, TR_DOWN ) );
     else if( !strcmp( key, "downloadLimit" ) )
@@ -427,16 +501,18 @@ addInfo( const tr_torrent * tor,
 }
 
 static const char*
-torrentGet( tr_handle * handle,
-            tr_benc *   args_in,
-            tr_benc *   args_out )
+torrentGet( tr_session               * session,
+            tr_benc                  * args_in,
+            tr_benc                  * args_out,
+            struct tr_rpc_idle_data  * idle_data )
 {
     int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( handle, args_in, &torrentCount );
-    tr_benc *     list = tr_bencDictAddList( args_out, "torrents",
-                                             torrentCount );
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+    tr_benc *     list = tr_bencDictAddList( args_out, "torrents", torrentCount );
     tr_benc *     fields;
-    char *        msg = NULL;
+    const char *  msg = NULL;
+
+    assert( idle_data == NULL );
 
     if( !tr_bencDictFindList( args_in, "fields", &fields ) )
         msg = "no fields specified";
@@ -451,23 +527,29 @@ torrentGet( tr_handle * handle,
 ****
 ***/
 
-static void
+static const char*
 setFilePriorities( tr_torrent * tor,
                    int          priority,
                    tr_benc *    list )
 {
-    int               i;
-    int64_t           tmp;
-    int               fileCount = 0;
-    const int         n = tr_bencListSize( list );
+    int i;
+    int64_t tmp;
+    int fileCount = 0;
+    const int n = tr_bencListSize( list );
+    const char * errmsg = NULL;
     tr_file_index_t * files = tr_new0( tr_file_index_t, tor->info.fileCount );
 
     if( n )
     {
-        for( i = 0; i < n; ++i )
-            if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) )
-                if( 0 <= tmp && tmp < tor->info.fileCount )
+        for( i = 0; i < n; ++i ) {
+            if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) ) {
+                if( 0 <= tmp && tmp < tor->info.fileCount ) {
                     files[fileCount++] = tmp;
+                } else {
+                    errmsg = "file index out of range";
+                }
+            }
+        }
     }
     else /* if empty set, apply to all */
     {
@@ -480,25 +562,32 @@ setFilePriorities( tr_torrent * tor,
         tr_torrentSetFilePriorities( tor, files, fileCount, priority );
 
     tr_free( files );
+    return errmsg;
 }
 
-static void
+static const char*
 setFileDLs( tr_torrent * tor,
             int          do_download,
             tr_benc *    list )
 {
-    int               i;
-    int64_t           tmp;
-    int               fileCount = 0;
-    const int         n = tr_bencListSize( list );
+    int i;
+    int64_t tmp;
+    int fileCount = 0;
+    const int n = tr_bencListSize( list );
+    const char * errmsg = NULL;
     tr_file_index_t * files = tr_new0( tr_file_index_t, tor->info.fileCount );
 
     if( n ) /* if argument list, process them */
     {
-        for( i = 0; i < n; ++i )
-            if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) )
-                if( 0 <= tmp && tmp < tor->info.fileCount )
+        for( i = 0; i < n; ++i ) {
+            if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) ) {
+                if( 0 <= tmp && tmp < tor->info.fileCount ) {
                     files[fileCount++] = tmp;
+                } else {
+                    errmsg = "file index out of range";
+                }
+            }
+        }
     }
     else /* if empty set, apply to all */
     {
@@ -511,15 +600,20 @@ setFileDLs( tr_torrent * tor,
         tr_torrentSetFileDLs( tor, files, fileCount, do_download );
 
     tr_free( files );
+    return errmsg;
 }
 
 static const char*
-torrentSet( tr_handle *        h,
-            tr_benc *          args_in,
-            tr_benc * args_out UNUSED )
+torrentSet( tr_session               * session,
+            tr_benc                  * args_in,
+            tr_benc                  * args_out UNUSED,
+            struct tr_rpc_idle_data  * idle_data )
 {
-    int           i, torrentCount;
-    tr_torrent ** torrents = getTorrents( h, args_in, &torrentCount );
+    const char * errmsg = NULL;
+    int i, torrentCount;
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+
+    assert( idle_data == NULL );
 
     for( i = 0; i < torrentCount; ++i )
     {
@@ -533,12 +627,12 @@ torrentSet( tr_handle *        h,
             setFileDLs( tor, TRUE, files );
         if( tr_bencDictFindInt( args_in, "peer-limit", &tmp ) )
             tr_torrentSetPeerLimit( tor, tmp );
-        if( tr_bencDictFindList( args_in, "priority-high", &files ) )
-            setFilePriorities( tor, TR_PRI_HIGH, files );
-        if( tr_bencDictFindList( args_in, "priority-low", &files ) )
-            setFilePriorities( tor, TR_PRI_LOW, files );
-        if( tr_bencDictFindList( args_in, "priority-normal", &files ) )
-            setFilePriorities( tor, TR_PRI_NORMAL, files );
+        if( !errmsg &&  tr_bencDictFindList( args_in, "priority-high", &files ) )
+            errmsg = setFilePriorities( tor, TR_PRI_HIGH, files );
+        if( !errmsg && tr_bencDictFindList( args_in, "priority-low", &files ) )
+            errmsg = setFilePriorities( tor, TR_PRI_LOW, files );
+        if( !errmsg && tr_bencDictFindList( args_in, "priority-normal", &files ) )
+            errmsg = setFilePriorities( tor, TR_PRI_NORMAL, files );
         if( tr_bencDictFindInt( args_in, "speed-limit-down", &tmp ) )
             tr_torrentSetSpeedLimit( tor, TR_DOWN, tmp );
         if( tr_bencDictFindInt( args_in, "speed-limit-down-enabled", &tmp ) )
@@ -551,25 +645,106 @@ torrentSet( tr_handle *        h,
             tr_torrentSetSpeedMode( tor, TR_UP, tmp ? TR_SPEEDLIMIT_SINGLE
                                     : TR_SPEEDLIMIT_GLOBAL );
 
-        notify( h, TR_RPC_TORRENT_CHANGED, tor );
+        notify( session, TR_RPC_TORRENT_CHANGED, tor );
     }
 
     tr_free( torrents );
-    return NULL;
+    return errmsg;
 }
 
 /***
 ****
 ***/
 
+static void
+addTorrentImpl( struct tr_rpc_idle_data * data, tr_ctor * ctor )
+{
+    int err = 0;
+    const char * result = NULL;
+    tr_torrent * tor = tr_torrentNew( data->session, ctor, &err );
+
+    tr_ctorFree( ctor );
+
+    if( tor )
+    {
+        tr_benc fields;
+        tr_bencInitList( &fields, 3 );
+        tr_bencListAddStr( &fields, "id" );
+        tr_bencListAddStr( &fields, "name" );
+        tr_bencListAddStr( &fields, "hashString" );
+        addInfo( tor, tr_bencDictAdd( data->args_out, "torrent-added" ), &fields );
+        notify( data->session, TR_RPC_TORRENT_ADDED, tor );
+        tr_bencFree( &fields );
+    }
+    else if( err == TR_EDUPLICATE )
+    {
+        result = "duplicate torrent";
+    }
+    else if( err == TR_EINVALID )
+    {
+        result = "invalid or corrupt torrent file";
+    }
+
+    tr_idle_function_done( data, result );
+}
+
+
+struct add_torrent_idle_data
+{
+    struct tr_rpc_idle_data * data;
+    tr_ctor * ctor;
+};
+
+static void
+gotMetadataFromURL( tr_session       * session UNUSED,
+                    long               response_code,
+                    const void       * response,
+                    size_t             response_byte_count,
+                    void             * user_data )
+{
+    struct add_torrent_idle_data * data = user_data;
+
+    dbgmsg( "torrentAdd: HTTP response code was %ld (%s); response length was %zu bytes",
+            response_code, tr_webGetResponseStr( response_code ), response_byte_count );
+
+    if( response_code == 200 )
+    {
+        tr_ctorSetMetainfo( data->ctor, response, response_byte_count );
+        addTorrentImpl( data->data, data->ctor );
+    }
+    else
+    {
+        char result[1024];
+        tr_snprintf( result, sizeof( result ), "http error %ld: %s",
+                     response_code, tr_webGetResponseStr( response_code ) );
+        tr_idle_function_done( data->data, result );
+    }
+
+    tr_free( data );
+}
+
+static tr_bool
+isCurlURL( const char * filename )
+{
+    if( filename == NULL )
+        return FALSE;
+
+    return ( strstr( filename, "ftp://" ) != NULL )
+        || ( strstr( filename, "http://" ) != NULL )
+        || ( strstr( filename, "https://" ) != NULL );
+}
+
 static const char*
-torrentAdd( tr_handle * h,
-            tr_benc *   args_in,
-            tr_benc *   args_out )
+torrentAdd( tr_session               * session,
+            tr_benc                  * args_in,
+            tr_benc                  * args_out UNUSED,
+            struct tr_rpc_idle_data  * idle_data )
 {
     const char * filename = NULL;
     const char * metainfo_base64 = NULL;
 
+    assert( idle_data != NULL );
+
     tr_bencDictFindStr( args_in, "filename", &filename );
     tr_bencDictFindStr( args_in, "metainfo", &metainfo_base64 );
     if( !filename && !metainfo_base64 )
@@ -577,23 +752,8 @@ torrentAdd( tr_handle * h,
     else
     {
         int64_t      i;
-        int          err = 0;
         const char * str;
-        tr_ctor *    ctor;
-        tr_torrent * tor;
-
-        ctor = tr_ctorNew( h );
-
-        /* set the metainfo */
-        if( filename )
-            tr_ctorSetMetainfoFromFile( ctor, filename );
-        else
-        {
-            int    len;
-            char * metainfo = tr_base64_decode( metainfo_base64, -1,  &len );
-            tr_ctorSetMetainfo( ctor, (uint8_t*)metainfo, len );
-            tr_free( metainfo );
-        }
+        tr_ctor    * ctor = tr_ctorNew( session );
 
         /* set the optional arguments */
         if( tr_bencDictFindStr( args_in, "download-dir", &str ) )
@@ -603,29 +763,28 @@ torrentAdd( tr_handle * h,
         if( tr_bencDictFindInt( args_in, "peer-limit", &i ) )
             tr_ctorSetPeerLimit( ctor, TR_FORCE, i );
 
-        tor = tr_torrentNew( h, ctor, &err );
-        tr_ctorFree( ctor );
+        dbgmsg( "torrentAdd: filename is \"%s\"", filename );
 
-        if( tor )
+        if( isCurlURL( filename ) )
         {
-            tr_benc fields;
-            tr_bencInitList( &fields, 3 );
-            tr_bencListAddStr( &fields, "id" );
-            tr_bencListAddStr( &fields, "name" );
-            tr_bencListAddStr( &fields, "hashString" );
-            addInfo( tor, tr_bencDictAdd( args_out,
-                                          "torrent-added" ), &fields );
-            notify( h, TR_RPC_TORRENT_ADDED, tor );
-            tr_bencFree( &fields );
+            struct add_torrent_idle_data * d = tr_new0( struct add_torrent_idle_data, 1 );
+            d->data = idle_data;
+            d->ctor = ctor;
+            tr_webRun( session, filename, NULL, gotMetadataFromURL, d );
         }
-        else if( err == TR_EDUPLICATE )
+        else
         {
-            return "duplicate torrent";
-        }
-        else if( err == TR_EINVALID )
-        {
-            return "invalid or corrupt torrent file";
+            if( filename != NULL )
+                tr_ctorSetMetainfoFromFile( ctor, filename );
+            else {
+                int len;
+                char * metainfo = tr_base64_decode( metainfo_base64, -1,  &len );
+                tr_ctorSetMetainfo( ctor, (uint8_t*)metainfo, len );
+                tr_free( metainfo );
+            }
+            addTorrentImpl( idle_data, ctor );
         }
+
     }
 
     return NULL;
@@ -636,109 +795,123 @@ torrentAdd( tr_handle * h,
 ***/
 
 static const char*
-sessionSet( tr_handle *        h,
-            tr_benc *          args_in,
-            tr_benc * args_out UNUSED )
+sessionSet( tr_session               * session,
+            tr_benc                  * args_in,
+            tr_benc                  * args_out UNUSED,
+            struct tr_rpc_idle_data  * idle_data )
 {
     int64_t      i;
     const char * str;
 
+    assert( idle_data == NULL );
+
     if( tr_bencDictFindStr( args_in, "download-dir", &str ) )
-        tr_sessionSetDownloadDir( h, str );
+        tr_sessionSetDownloadDir( session, str );
     if( tr_bencDictFindInt( args_in, "peer-limit", &i ) )
-        tr_sessionSetPeerLimit( h, i );
+        tr_sessionSetPeerLimit( session, i );
     if( tr_bencDictFindInt( args_in, "pex-allowed", &i ) )
-        tr_sessionSetPexEnabled( h, i );
+        tr_sessionSetPexEnabled( session, i );
     if( tr_bencDictFindInt( args_in, "port", &i ) )
-        tr_sessionSetPeerPort( h, i );
+        tr_sessionSetPeerPort( session, i );
     if( tr_bencDictFindInt( args_in, "port-forwarding-enabled", &i ) )
-        tr_sessionSetPortForwardingEnabled( h, i );
+        tr_sessionSetPortForwardingEnabled( session, i );
     if( tr_bencDictFindInt( args_in, "speed-limit-down", &i ) )
-        tr_sessionSetSpeedLimit( h, TR_DOWN, i );
+        tr_sessionSetSpeedLimit( session, TR_DOWN, i );
     if( tr_bencDictFindInt( args_in, "speed-limit-down-enabled", &i ) )
-        tr_sessionSetSpeedLimitEnabled( h, TR_DOWN, i );
+        tr_sessionSetSpeedLimitEnabled( session, TR_DOWN, i );
     if( tr_bencDictFindInt( args_in, "speed-limit-up", &i ) )
-        tr_sessionSetSpeedLimit( h, TR_UP, i );
+        tr_sessionSetSpeedLimit( session, TR_UP, i );
     if( tr_bencDictFindInt( args_in, "speed-limit-up-enabled", &i ) )
-        tr_sessionSetSpeedLimitEnabled( h, TR_UP, i );
+        tr_sessionSetSpeedLimitEnabled( session, TR_UP, i );
     if( tr_bencDictFindStr( args_in, "encryption", &str ) )
     {
         if( !strcmp( str, "required" ) )
-            tr_sessionSetEncryption( h, TR_ENCRYPTION_REQUIRED );
+            tr_sessionSetEncryption( session, TR_ENCRYPTION_REQUIRED );
         else if( !strcmp( str, "tolerated" ) )
-            tr_sessionSetEncryption( h, TR_CLEAR_PREFERRED );
+            tr_sessionSetEncryption( session, TR_CLEAR_PREFERRED );
         else
-            tr_sessionSetEncryption( h, TR_ENCRYPTION_PREFERRED );
+            tr_sessionSetEncryption( session, TR_ENCRYPTION_PREFERRED );
     }
 
-    notify( h, TR_RPC_SESSION_CHANGED, NULL );
+    notify( session, TR_RPC_SESSION_CHANGED, NULL );
 
     return NULL;
 }
 
 static const char*
-sessionStats( tr_handle *       h,
-              tr_benc * args_in UNUSED,
-              tr_benc *         args_out )
+sessionStats( tr_session               * session,
+              tr_benc                  * args_in UNUSED,
+              tr_benc                  * args_out,
+              struct tr_rpc_idle_data  * idle_data )
 {
-    tr_benc *    d = tr_bencDictAddDict( args_out, "session-stats", 10 );
+    int running = 0;
+    int total = 0;
+    tr_benc * d;  
+    tr_session_stats currentStats = { 0.0f, 0, 0, 0, 0, 0 }; 
+    tr_session_stats cumulativeStats = { 0.0f, 0, 0, 0, 0, 0 }; 
     tr_torrent * tor = NULL;
-    int          running = 0;
-    int          total = 0;
 
-    while( ( tor = tr_torrentNext( h, tor ) ) )
-    {
+    assert( idle_data == NULL );
+
+    while(( tor = tr_torrentNext( session, tor ))) {
         ++total;
         if( tor->isRunning )
             ++running;
     }
 
-    tr_bencDictAddInt( d, "activeTorrentCount", running );
-    tr_bencDictAddInt( d, "downloadSpeed", (int)( tr_sessionGetPieceSpeed( h, TR_DOWN ) * 1024 ) );
-    tr_bencDictAddInt( d, "pausedTorrentCount", total - running );
-    tr_bencDictAddInt( d, "torrentCount", total );
-    tr_bencDictAddInt( d, "uploadSpeed", (int)( tr_sessionGetPieceSpeed( h, TR_UP ) * 1024 ) );
+    tr_sessionGetStats( session, &currentStats ); 
+    tr_sessionGetCumulativeStats( session, &cumulativeStats ); 
+
+    tr_bencDictAddInt( args_out, "activeTorrentCount", running );
+    tr_bencDictAddInt( args_out, "downloadSpeed", (int)( tr_sessionGetPieceSpeed( session, TR_DOWN ) * 1024 ) );
+    tr_bencDictAddInt( args_out, "pausedTorrentCount", total - running );
+    tr_bencDictAddInt( args_out, "torrentCount", total );
+    tr_bencDictAddInt( args_out, "uploadSpeed", (int)( tr_sessionGetPieceSpeed( session, TR_UP ) * 1024 ) );
+
+    d = tr_bencDictAddDict( args_out, "cumulative-stats", 5 );  
+    tr_bencDictAddInt( d, "downloadedBytes", cumulativeStats.downloadedBytes ); 
+    tr_bencDictAddInt( d, "filesAdded", cumulativeStats.filesAdded ); 
+    tr_bencDictAddInt( d, "secondsActive", cumulativeStats.secondsActive ); 
+    tr_bencDictAddInt( d, "sessionCount", cumulativeStats.sessionCount ); 
+    tr_bencDictAddInt( d, "uploadedBytes", cumulativeStats.uploadedBytes ); 
+
+    d = tr_bencDictAddDict( args_out, "current-stats", 5 );  
+    tr_bencDictAddInt( d, "downloadedBytes", currentStats.downloadedBytes ); 
+    tr_bencDictAddInt( d, "filesAdded", currentStats.filesAdded ); 
+    tr_bencDictAddInt( d, "secondsActive", currentStats.secondsActive ); 
+    tr_bencDictAddInt( d, "sessionCount", currentStats.sessionCount ); 
+    tr_bencDictAddInt( d, "uploadedBytes", currentStats.uploadedBytes ); 
+
     return NULL;
 }
 
 static const char*
-sessionGet( tr_handle *       h,
-            tr_benc * args_in UNUSED,
-            tr_benc *         args_out )
+sessionGet( tr_session               * session,
+            tr_benc                  * args_in UNUSED,
+            tr_benc                  * args_out,
+            struct tr_rpc_idle_data  * idle_data )
 {
     const char * str;
     tr_benc *    d = args_out;
 
-    tr_bencDictAddStr( d, "download-dir",
-                      tr_sessionGetDownloadDir( h ) );
-    tr_bencDictAddInt( d, "peer-limit",
-                      tr_sessionGetPeerLimit( h ) );
-    tr_bencDictAddInt( d, "pex-allowed",
-                      tr_sessionIsPexEnabled( h ) );
-    tr_bencDictAddInt( d, "port",
-                      tr_sessionGetPeerPort( h ) );
-    tr_bencDictAddInt( d, "port-forwarding-enabled",
-                      tr_sessionIsPortForwardingEnabled( h ) );
-    tr_bencDictAddInt( d, "speed-limit-up",
-                       tr_sessionGetSpeedLimit( h, TR_UP ) );
-    tr_bencDictAddInt( d, "speed-limit-up-enabled",
-                      tr_sessionIsSpeedLimitEnabled( h, TR_UP ) );
-    tr_bencDictAddInt( d, "speed-limit-down",
-                       tr_sessionGetSpeedLimit( h, TR_DOWN ) );
-    tr_bencDictAddInt( d, "speed-limit-down-enabled",
-                      tr_sessionIsSpeedLimitEnabled( h, TR_DOWN ) );
+    assert( idle_data == NULL );
+
+    tr_bencDictAddStr( d, "download-dir", tr_sessionGetDownloadDir( session ) );
+    tr_bencDictAddInt( d, "peer-limit", tr_sessionGetPeerLimit( session ) );
+    tr_bencDictAddInt( d, "pex-allowed", tr_sessionIsPexEnabled( session ) );
+    tr_bencDictAddInt( d, "port", tr_sessionGetPeerPort( session ) );
+    tr_bencDictAddInt( d, "port-forwarding-enabled", tr_sessionIsPortForwardingEnabled( session ) );
+    tr_bencDictAddInt( d, "rpc-version", 4 );
+    tr_bencDictAddInt( d, "rpc-version-minimum", 1 );
+    tr_bencDictAddInt( d, "speed-limit-up", tr_sessionGetSpeedLimit( session, TR_UP ) );
+    tr_bencDictAddInt( d, "speed-limit-up-enabled", tr_sessionIsSpeedLimitEnabled( session, TR_UP ) );
+    tr_bencDictAddInt( d, "speed-limit-down", tr_sessionGetSpeedLimit( session, TR_DOWN ) );
+    tr_bencDictAddInt( d, "speed-limit-down-enabled", tr_sessionIsSpeedLimitEnabled( session, TR_DOWN ) );
     tr_bencDictAddStr( d, "version", LONG_VERSION_STRING );
-
-    switch( tr_sessionGetEncryption( h ) )
-    {
-        case TR_CLEAR_PREFERRED:
-            str = "tolerated"; break;
-
-        case TR_ENCRYPTION_REQUIRED:
-            str = "required"; break;
-
-        default:
-            str = "preferred"; break;
+    switch( tr_sessionGetEncryption( session ) ) {
+        case TR_CLEAR_PREFERRED: str = "tolerated"; break;
+        case TR_ENCRYPTION_REQUIRED: str = "required"; break; 
+        default: str = "preferred"; break;
     }
     tr_bencDictAddStr( d, "encryption", str );
 
@@ -749,109 +922,137 @@ sessionGet( tr_handle *       h,
 ****
 ***/
 
-typedef const char* ( handler )( tr_handle*, tr_benc*, tr_benc* );
+typedef const char* ( *handler )( tr_session*, tr_benc*, tr_benc*, struct tr_rpc_idle_data * );
 
 static struct method
 {
     const char *  name;
-    handler *     func;
-} methods[] = {
-    { "session-get",    sessionGet          },
-    { "session-set",    sessionSet          },
-    { "session-stats",  sessionStats        },
-    { "torrent-add",    torrentAdd          },
-    { "torrent-get",    torrentGet          },
-    { "torrent-remove", torrentRemove       },
-    { "torrent-set",    torrentSet          },
-    { "torrent-start",  torrentStart        },
-    { "torrent-stop",   torrentStop         },
-    { "torrent-verify", torrentVerify       }
+    tr_bool       immediate;
+    handler       func;
+}
+methods[] =
+{
+    { "session-get",    TRUE,  sessionGet          },
+    { "session-set",    TRUE,  sessionSet          },
+    { "session-stats",  TRUE,  sessionStats        },
+    { "torrent-add",    FALSE, torrentAdd          },
+    { "torrent-get",    TRUE,  torrentGet          },
+    { "torrent-remove", TRUE,  torrentRemove       },
+    { "torrent-set",    TRUE,  torrentSet          },
+    { "torrent-start",  TRUE,  torrentStart        },
+    { "torrent-stop",   TRUE,  torrentStop         },
+    { "torrent-verify", TRUE,  torrentVerify       }
 };
 
-static char*
-request_exec( struct tr_handle * handle,
-              tr_benc *          request,
-              int *              response_len )
+static void
+noop_response_callback( tr_session * session UNUSED,
+                        const char * response UNUSED,
+                        size_t       response_len UNUSED,
+                        void       * user_data UNUSED )
 {
-    int64_t      i;
+}
+
+static void
+request_exec( tr_session             * session,
+              tr_benc                * request,
+              tr_rpc_response_func     callback,
+              void                   * callback_user_data )
+{
+    int i;
     const char * str;
-    char *       out;
-    tr_benc      response;
-    tr_benc *    args_in = tr_bencDictFind( request, "arguments" );
-    tr_benc *    args_out = NULL;
+    tr_benc * args_in = tr_bencDictFind( request, "arguments" );
     const char * result = NULL;
 
-    /* build the response skeleton */
-    tr_bencInitDict( &response, 3 );
-    args_out = tr_bencDictAddDict( &response, "arguments", 0 );
+    if( callback == NULL )
+        callback = noop_response_callback;
 
     /* parse the request */
     if( !tr_bencDictFindStr( request, "method", &str ) )
         result = "no method name";
-    else
-    {
+    else {
         const int n = TR_N_ELEMENTS( methods );
         for( i = 0; i < n; ++i )
             if( !strcmp( str, methods[i].name ) )
                 break;
-        result = i == n
-                 ? "method name not recognized"
-                 : ( *methods[i].func )( handle, args_in, args_out );
+        if( i ==n )
+            result = "method name not recognized";
     }
 
-    /* serialize & return the response */
-    if( !result )
-        result = "success";
-    tr_bencDictAddStr( &response, "result", result );
-    if( tr_bencDictFindInt( request, "tag", &i ) )
-        tr_bencDictAddInt( &response, "tag", i );
-    out = tr_bencSaveAsJSON( &response, response_len );
-    tr_bencFree( &response );
-    return out;
+    /* if we couldn't figure out which method to use, return an error */
+    if( result != NULL )
+    {
+        int64_t tag;
+        tr_benc response;
+        struct evbuffer * buf = tr_getBuffer( );
+
+        tr_bencInitDict( &response, 3 );
+        tr_bencDictAddDict( &response, "arguments", 0 );
+        tr_bencDictAddStr( &response, "result", result );
+        if( tr_bencDictFindInt( request, "tag", &tag ) )
+            tr_bencDictAddInt( &response, "tag", tag );
+        tr_bencSaveAsJSON( &response, buf );
+        (*callback)( session, (const char*)EVBUFFER_DATA(buf),
+                     EVBUFFER_LENGTH( buf ), callback_user_data );
+
+        tr_releaseBuffer( buf );
+        tr_bencFree( &response );
+    }
+    else if( methods[i].immediate )
+    {
+        int64_t tag;
+        tr_benc response;
+        tr_benc * args_out;
+        struct evbuffer * buf = tr_getBuffer( );
+
+        tr_bencInitDict( &response, 3 );
+        args_out = tr_bencDictAddDict( &response, "arguments", 0 );
+        result = (*methods[i].func)( session, args_in, args_out, NULL );
+        if( result == NULL )
+            result = "success";
+        tr_bencDictAddStr( &response, "result", result );
+        if( tr_bencDictFindInt( request, "tag", &tag ) )
+            tr_bencDictAddInt( &response, "tag", tag );
+        tr_bencSaveAsJSON( &response, buf );
+        (*callback)( session, (const char*)EVBUFFER_DATA(buf),
+                     EVBUFFER_LENGTH(buf), callback_user_data );
+
+        tr_releaseBuffer( buf );
+        tr_bencFree( &response );
+    }
+    else
+    {
+        int64_t tag;
+        struct tr_rpc_idle_data * data = tr_new0( struct tr_rpc_idle_data, 1 );
+        data->session = session;
+        data->response = tr_new0( tr_benc, 1 );
+        tr_bencInitDict( data->response, 3 );
+        if( tr_bencDictFindInt( request, "tag", &tag ) )
+            tr_bencDictAddInt( data->response, "tag", tag );
+        data->args_out = tr_bencDictAddDict( data->response, "arguments", 0 );
+        data->callback = callback;
+        data->callback_user_data = callback_user_data;
+        (*methods[i].func)( session, args_in, data->args_out, data );
+    }
 }
 
-char*
-tr_rpc_request_exec_json( struct tr_handle * handle,
-                          const void *       request_json,
-                          int                request_len,
-                          int *              response_len )
+void
+tr_rpc_request_exec_json( tr_session            * session,
+                          const void            * request_json,
+                          int                     request_len,
+                          tr_rpc_response_func    callback,
+                          void                  * callback_user_data )
 {
     tr_benc top;
-    int     have_content;
-    char *  ret;
+    int have_content;
 
     if( request_len < 0 )
         request_len = strlen( request_json );
 
     have_content = !tr_jsonParse( request_json, request_len, &top, NULL );
-    ret = request_exec( handle, have_content ? &top : NULL, response_len );
+    request_exec( session, have_content ? &top : NULL, callback, callback_user_data );
 
     if( have_content )
         tr_bencFree( &top );
-    return ret;
-}
-
-static void
-addToken( tr_benc *    list,
-          const char * token,
-          size_t       len )
-{
-    char *       p;
-    const char * end = token + len;
-    const long   a = strtol( token, &p, 10 );
-
-    if( p == end )
-        tr_bencListAddInt( list, a );
-    else if( *p == '-' && isdigit( p[1] ) )
-    {
-        const long b = strtol( p + 1, &p, 10 );
-        if( ( p == end ) && ( b > a ) )
-        {
-            long i;
-            for( i = a; i <= b; ++i )
-                tr_bencListAddInt( list, i );
-        }
-    }
 }
 
 /**
@@ -864,59 +1065,35 @@ addToken( tr_benc *    list,
  * - all other values are strings
  */
 void
-tr_rpc_parse_list_str( tr_benc *    setme,
-                       const char * str_in,
-                       size_t       len )
+tr_rpc_parse_list_str( tr_benc     * setme,
+                       const char  * str,
+                       int           len )
 
 {
-    char *       str = tr_strndup( str_in, len );
-    int          isNum;
-    int          isNumList;
-    int          commaCount;
-    const char * walk;
+    int valueCount;
+    int * values = tr_parseNumberRange( str, len, &valueCount );
 
-    isNum = 1;
-    isNumList = 1;
-    commaCount = 0;
-    walk = str;
-    for( ; *walk && ( isNumList || isNum ); ++walk )
-    {
-        if( isNumList ) isNumList = *walk == '-' || isdigit( *walk )
-                                    || *walk == ',';
-        if( isNum     ) isNum     = *walk == '-' || isdigit( *walk );
-        if( *walk == ',' ) ++commaCount;
-    }
-
-    if( isNum )
-        tr_bencInitInt( setme, strtol( str, NULL, 10 ) );
-    else if( !isNumList )
+    if( valueCount == 0 )
         tr_bencInitStr( setme, str, len );
-    else
-    {
-        tr_bencInitList( setme, commaCount + 1 );
-        walk = str;
-        while( *walk )
-        {
-            const char * p = strchr( walk, ',' );
-            if( !p )
-                p = walk + strlen( walk );
-            addToken( setme, walk, p - walk );
-            if( *p != ',' )
-                break;
-            walk = p + 1;
-        }
+    else if( valueCount == 1 )
+        tr_bencInitInt( setme, values[0] );
+    else {
+        int i;
+        tr_bencInitList( setme, valueCount );
+        for( i=0; i<valueCount; ++i )
+            tr_bencListAddInt( setme, values[i] );
     }
 
-    tr_free( str );
+    tr_free( values );
 }
 
-char*
-tr_rpc_request_exec_uri( struct tr_handle * handle,
-                         const void *       request_uri,
-                         int                request_len,
-                         int *              response_len )
+void
+tr_rpc_request_exec_uri( tr_session           * session,
+                         const void           * request_uri,
+                         int                    request_len,
+                         tr_rpc_response_func   callback,
+                         void                 * callback_user_data )
 {
-    char *       ret = NULL;
     tr_benc      top, * args;
     char *       request = tr_strndup( request_uri, request_len );
     const char * pch;
@@ -945,11 +1122,9 @@ tr_rpc_request_exec_uri( struct tr_handle * handle,
         pch = next ? next + 1 : NULL;
     }
 
-    ret = request_exec( handle, &top, response_len );
+    request_exec( session, &top, callback, callback_user_data );
 
     /* cleanup */
     tr_bencFree( &top );
     tr_free( request );
-    return ret;
 }
-
