@@ -1,43 +1,37 @@
-/******************************************************************************
- * $Id: torrent.h 7365 2008-12-13 22:52:13Z charles $
+/*
+ * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license. 
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: torrent.h 8218 2009-04-12 15:01:29Z charles $
+ */
 
 #ifndef __TRANSMISSION__
-#error only libtransmission should #include this header.
+ #error only libtransmission should #include this header.
 #endif
 
 #ifndef TR_TORRENT_H
 #define TR_TORRENT_H 1
 
+#include "completion.h" /* tr_completion */
+#include "ratecontrol.h" /* tr_ratecontrol */
+#include "session.h" /* tr_globalLock(), tr_globalUnlock() */
+#include "utils.h" /* tr_bitfield */
+
 struct tr_bandwidth;
 struct tr_ratecontrol;
+struct tr_torrent_peers;
 
 /**
 ***  Package-visible ctor API
 **/
 
 void        tr_ctorSetSave( tr_ctor * ctor,
-                            int       saveMetadataInOurTorrentsDir );
+                            tr_bool   saveMetadataInOurTorrentsDir );
 
 int         tr_ctorGetSave( const tr_ctor * ctor );
 
@@ -49,9 +43,7 @@ int         tr_ctorGetSave( const tr_ctor * ctor );
 void        tr_torrentInitFileDLs( tr_torrent *      tor,
                                    tr_file_index_t * files,
                                    tr_file_index_t   fileCount,
-                                   int               do_download );
-
-int         tr_torrentIsPrivate( const tr_torrent * );
+                                   tr_bool           do_download );
 
 void        tr_torrentRecheckCompleteness( tr_torrent * );
 
@@ -59,19 +51,10 @@ void        tr_torrentResetTransferStats( tr_torrent * );
 
 void        tr_torrentSetHasPiece( tr_torrent *     tor,
                                    tr_piece_index_t pieceIndex,
-                                   int              has );
-
-void        tr_torrentLock( const tr_torrent * session );
-
-void        tr_torrentUnlock( const tr_torrent * session );
-
-int         tr_torrentIsSeed( const tr_torrent * session );
+                                   tr_bool          has );
 
 void        tr_torrentChangeMyPort( tr_torrent * session );
 
-int         tr_torrentExists( const tr_session * session,
-                              const uint8_t    * hash );
-
 tr_torrent* tr_torrentFindFromId( tr_session * session,
                                   int          id );
 
@@ -84,43 +67,16 @@ tr_torrent* tr_torrentFindFromHashString( tr_session * session,
 tr_torrent* tr_torrentFindFromObfuscatedHash( tr_session    * session,
                                               const uint8_t * hash );
 
-int         tr_torrentAllowsPex( const tr_torrent * );
-
 tr_bool     tr_torrentIsPieceTransferAllowed( const tr_torrent * torrent,
                                               tr_direction       direction );
 
-/* get the index of this piece's first block */
-#define tr_torPieceFirstBlock( tor, piece ) ( ( piece ) *\
-                                             ( tor )->blockCountInPiece )
-
-/* what piece index is this block in? */
-#define tr_torBlockPiece( tor, block ) ( ( block ) /\
-                                        ( tor )->blockCountInPiece )
-
-/* how many blocks are in this piece? */
-#define tr_torPieceCountBlocks( tor, piece ) \
-    ( ( ( piece ) ==\
-       ( ( tor )->info.pieceCount - \
-        1 ) ) ? ( tor )->blockCountInLastPiece : ( tor )->blockCountInPiece )
-
-/* how many bytes are in this piece? */
-#define tr_torPieceCountBytes( tor, piece ) \
-    ( ( ( piece ) ==\
-       ( ( tor )->info.pieceCount - \
-        1 ) ) ? ( tor )->lastPieceSize : ( tor )->info.pieceSize )
-
-/* how many bytes are in this block? */
-#define tr_torBlockCountBytes( tor, block ) \
-    ( ( ( block ) ==\
-       ( ( tor )->blockCount - \
-        1 ) ) ? ( tor )->lastBlockSize : ( tor )->blockSize )
 
 #define tr_block( a, b ) _tr_block( tor, a, b )
 tr_block_index_t _tr_block( const tr_torrent * tor,
                             tr_piece_index_t   index,
                             uint32_t           offset );
 
-int              tr_torrentReqIsValid( const tr_torrent * tor,
+tr_bool          tr_torrentReqIsValid( const tr_torrent * tor,
                                        tr_piece_index_t   index,
                                        uint32_t           offset,
                                        uint32_t           length );
@@ -137,19 +93,16 @@ void             tr_torrentInitFilePriority( tr_torrent       * tor,
 
 int              tr_torrentCountUncheckedPieces( const tr_torrent * );
 
-int              tr_torrentIsPieceChecked( const tr_torrent  * tor,
-                                           tr_piece_index_t    piece );
-
-int              tr_torrentIsFileChecked( const tr_torrent  * tor,
+tr_bool          tr_torrentIsFileChecked( const tr_torrent  * tor,
                                           tr_file_index_t     file );
 
 void             tr_torrentSetPieceChecked( tr_torrent       * tor,
                                             tr_piece_index_t   piece,
-                                            int                isChecked );
+                                            tr_bool            isChecked );
 
 void             tr_torrentSetFileChecked( tr_torrent       * tor,
                                            tr_file_index_t    file,
-                                           int                isChecked );
+                                           tr_bool            isChecked );
 
 void             tr_torrentUncheck( tr_torrent * tor );
 
@@ -159,6 +112,10 @@ int              tr_torrentPromoteTracker( tr_torrent   * tor,
 time_t*          tr_torrentGetMTimes( const tr_torrent  * tor,
                                       size_t            * setmeCount );
 
+tr_torrent*      tr_torrentNext( tr_session  * session,
+                                 tr_torrent  * current );
+
+
 typedef enum
 {
     TR_VERIFY_NONE,
@@ -174,7 +131,9 @@ struct tr_torrent
 
     tr_speedlimit            speedLimitMode[2];
 
-    struct tr_ratecontrol *  swarmSpeed;
+    struct tr_ratecontrol    swarmSpeed;
+
+    int                      magicNumber;
 
     int                      error;
     char                     errorString[128];
@@ -203,9 +162,9 @@ struct tr_torrent
     uint32_t                   blockCountInPiece;
     uint32_t                   blockCountInLastPiece;
 
-    struct tr_completion *     completion;
+    struct tr_completion       completion;
 
-    struct tr_bitfield *       checkedPieces;
+    struct tr_bitfield         checkedPieces;
     tr_completeness            completeness;
 
     struct tr_tracker *        tracker;
@@ -241,6 +200,99 @@ struct tr_torrent
     int                        uniqueId;
 
     struct tr_bandwidth      * bandwidth;
+
+    struct tr_torrent_peers  * torrentPeers;
 };
 
+/* get the index of this piece's first block */
+static TR_INLINE tr_block_index_t
+tr_torPieceFirstBlock( const tr_torrent * tor, const tr_piece_index_t piece )
+{
+    return piece * tor->blockCountInPiece;
+}
+
+/* what piece index is this block in? */
+static TR_INLINE tr_piece_index_t
+tr_torBlockPiece( const tr_torrent * tor, const tr_block_index_t block )
+{
+    return block / tor->blockCountInPiece;
+}
+
+/* how many blocks are in this piece? */
+static TR_INLINE uint32_t
+tr_torPieceCountBlocks( const tr_torrent * tor, const tr_piece_index_t piece )
+{
+    return piece == tor->info.pieceCount - 1 ? tor->blockCountInLastPiece
+                                             : tor->blockCountInPiece;
+}
+
+/* how many bytes are in this piece? */
+static TR_INLINE uint32_t
+tr_torPieceCountBytes( const tr_torrent * tor, const tr_piece_index_t piece )
+{
+    return piece == tor->info.pieceCount - 1 ? tor->lastPieceSize
+                                             : tor->info.pieceSize;
+}
+
+/* how many bytes are in this block? */
+static TR_INLINE uint32_t
+tr_torBlockCountBytes( const tr_torrent * tor, const tr_block_index_t block )
+{
+    return block == tor->blockCount - 1 ? tor->lastBlockSize
+                                        : tor->blockSize;
+}
+
+static TR_INLINE void tr_torrentLock( const tr_torrent * tor )
+{
+    tr_globalLock( tor->session );
+}
+
+static TR_INLINE void tr_torrentUnlock( const tr_torrent * tor )
+{
+    tr_globalUnlock( tor->session );
+}
+
+static TR_INLINE tr_bool
+tr_torrentExists( const tr_session * session, const uint8_t *   torrentHash )
+{
+    return tr_torrentFindFromHash( (tr_session*)session, torrentHash ) != NULL;
+}
+
+static TR_INLINE tr_bool
+tr_torrentIsSeed( const tr_torrent * tor )
+{
+    return tor->completeness != TR_LEECH;
+}
+
+static TR_INLINE tr_bool tr_torrentIsPrivate( const tr_torrent * tor )
+{
+    return ( tor != NULL ) && tor->info.isPrivate;
+}
+
+static TR_INLINE tr_bool tr_torrentAllowsPex( const tr_torrent * tor )
+{
+    return ( tor != NULL  ) && tor->session->isPexEnabled && !tr_torrentIsPrivate( tor );
+}
+
+static TR_INLINE tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor, tr_piece_index_t i )
+{
+    return tr_bitfieldHasFast( &tor->checkedPieces, i );
+}
+
+/***
+****
+***/
+
+enum
+{
+    TORRENT_MAGIC_NUMBER = 95549
+};
+
+static inline tr_bool tr_isTorrent( const tr_torrent * tor )
+{
+    return ( tor != NULL )
+        && ( tor->magicNumber == TORRENT_MAGIC_NUMBER )
+        && ( tr_isSession( tor->session ) );
+}
+
 #endif
