@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tracker.c 7722 2009-01-16 17:31:10Z charles $
+ * $Id: tracker.c 8204 2009-04-10 17:34:25Z charles $
  */
 
 #include <assert.h>
@@ -259,7 +259,6 @@ publishNewPeersCompact( tr_tracker * t,
     {
         memcpy( &addr.addr.addr4, compactWalk, 4 );
         memcpy( &port, compactWalk + 4, 2 );
-        tr_suspectAddress( &addr, "compact" );
         
         memcpy( walk, &addr, sizeof( addr ) );
         memcpy( walk + sizeof( addr ), &port, 2 );
@@ -292,7 +291,6 @@ publishNewPeersCompact6( tr_tracker * t,
         memcpy( &addr.addr.addr6, compactWalk, 16 );
         memcpy( &port, compactWalk + 16, 2 );
         compactWalk += 18;
-        tr_suspectAddress( &addr, "compact6" );
         
         memcpy( walk, &addr, sizeof( addr ) );
         memcpy( walk + sizeof( addr ), &port, 2 );
@@ -346,7 +344,7 @@ parseOldPeers( tr_benc * bePeers,
     uint8_t * array, *walk;
     const int peerCount = bePeers->val.l.count;
 
-    assert( bePeers->type == TYPE_LIST );
+    assert( tr_bencIsList( bePeers ) );
 
     array = tr_new( uint8_t, peerCount * ( sizeof( tr_address ) + 2 ) );
 
@@ -368,7 +366,6 @@ parseOldPeers( tr_benc * bePeers,
             continue;
 
         memcpy( walk, &addr, sizeof( tr_address ) );
-        tr_suspectAddress( &addr, "old tracker" );
         port = htons( itmp );
         memcpy( walk + sizeof( tr_address ), &port, 2 );
         walk += sizeof( tr_address ) + 2;
@@ -484,12 +481,12 @@ onTrackerResponse( tr_session * session,
             {
                 const int allAreSeeds = incomplete == 0;
 
-                if( tmp->type == TYPE_STR ) /* "compact" extension */
+                if( tr_bencIsString( tmp ) ) /* "compact" extension */
                 {
                     publishNewPeersCompact( t, allAreSeeds, tmp->val.s.s,
                                             tmp->val.s.i );
                 }
-                else if( tmp->type == TYPE_LIST ) /* original protocol */
+                else if( tr_bencIsList( tmp ) ) /* original protocol */
                 {
                     size_t    byteCount = 0;
                     uint8_t * array = parseOldPeers( tmp, &byteCount );
@@ -502,7 +499,7 @@ onTrackerResponse( tr_session * session,
             {
                 const int allAreSeeds = incomplete == 0;
                 
-                if( tmp->type == TYPE_STR ) /* "compact" extension */
+                if( tr_bencIsString( tmp ) ) /* "compact" extension */
                 {
                     publishNewPeersCompact6( t, allAreSeeds, tmp->val.s.s,
                                              tmp->val.s.i );
@@ -661,7 +658,7 @@ onScrapeResponse( tr_session * session,
                     t->scrapeIntervalSec = DEFAULT_SCRAPE_INTERVAL_SEC;
 
                 tr_ndbg( t->name,
-                         "Scrape successful.  Rescraping in %d seconds.",
+                         "Scrape successful. Rescraping in %d seconds.",
                          t->scrapeIntervalSec );
 
                 success = TRUE;
@@ -685,16 +682,16 @@ onScrapeResponse( tr_session * session,
     if( 200 <= responseCode && responseCode <= 299 )
     {
         const int interval = t->scrapeIntervalSec + t->randOffset;
-        dbgmsg( t->name, "request succeeded. rescraping in %d seconds",
+        dbgmsg( t->name, "Request succeeded. Rescraping in %d seconds",
                 interval );
-        tr_ndbg( t->name, "request succeeded. rescraping in %d seconds",
+        tr_ndbg( t->name, "Request succeeded. Rescraping in %d seconds",
                  interval );
         t->scrapeAt = time( NULL ) + interval;
     }
     else if( 300 <= responseCode && responseCode <= 399 )
     {
         const int interval = 5;
-        dbgmsg( t->name, "got a redirect. retrying in %d seconds", interval );
+        dbgmsg( t->name, "Got a redirect. Retrying in %d seconds", interval );
         t->scrapeAt = time( NULL ) + interval;
     }
     else
@@ -702,7 +699,7 @@ onScrapeResponse( tr_session * session,
         const int interval = t->retryScrapeIntervalSec + t->randOffset;
         dbgmsg(
             t->name,
-            "Tracker responded to scrape with %ld.  Retrying in %d seconds.",
+            "Tracker responded to scrape with %ld. Retrying in %d seconds.",
             responseCode,  interval );
         t->retryScrapeIntervalSec *= 2;
         t->scrapeAt = time( NULL ) + interval;
@@ -720,7 +717,8 @@ enum
     TR_REQ_STOPPED,
     TR_REQ_PAUSED,     /* BEP 21 */
     TR_REQ_REANNOUNCE,
-    TR_REQ_SCRAPE
+    TR_REQ_SCRAPE,
+    TR_NUM_REQ_TYPES
 };
 
 struct tr_tracker_request
@@ -835,27 +833,33 @@ createScrape( tr_session * session,
 
 struct tr_tracker_handle
 {
+    tr_bool     shutdownHint;
     int         runningCount;
     tr_timer *  pulseTimer;
 };
 
 static int trackerPulse( void * vsession );
 
-static void
-ensureGlobalsExist( tr_session * session )
+void
+tr_trackerSessionInit( tr_session * session )
 {
-    if( session->tracker == NULL )
-    {
-        session->tracker = tr_new0( struct tr_tracker_handle, 1 );
-        session->tracker->pulseTimer =
-            tr_timerNew( session, trackerPulse, session,
-                         PULSE_INTERVAL_MSEC );
-        dbgmsg( NULL, "creating tracker timer" );
-    }
+    assert( tr_isSession( session ) );
+
+    session->tracker = tr_new0( struct tr_tracker_handle, 1 );
+    session->tracker->pulseTimer = tr_timerNew( session, trackerPulse, session, PULSE_INTERVAL_MSEC );
+    dbgmsg( NULL, "creating tracker timer" );
 }
 
 void
 tr_trackerSessionClose( tr_session * session )
+{
+    assert( tr_isSession( session ) );
+
+    session->tracker->shutdownHint = TRUE;
+}
+
+static void
+tr_trackerSessionDestroy( tr_session * session )
 {
     if( session && session->tracker )
     {
@@ -874,9 +878,19 @@ static void
 invokeRequest( void * vreq )
 {
     struct tr_tracker_request * req = vreq;
-    tr_tracker * t = findTracker( req->session, req->torrentId );
+    tr_tracker * t;
 
-    if( t )
+    assert( req != NULL );
+    assert( tr_isSession( req->session ) );
+    assert( req->torrentId >= 0 );
+    assert( req->reqtype >= 0 );
+    assert( req->reqtype < TR_NUM_REQ_TYPES );
+
+    dbgmsg( NULL, "invokeRequest got session %p, tracker %p", req->session, req->session->tracker );
+
+    t = findTracker( req->session, req->torrentId );
+
+    if( t != NULL )
     {
         const time_t now = time( NULL );
 
@@ -893,6 +907,7 @@ invokeRequest( void * vreq )
         }
     }
 
+    assert( req->session->tracker != NULL );
     ++req->session->tracker->runningCount;
 
     tr_webRun( req->session,
@@ -907,7 +922,10 @@ static void
 enqueueScrape( tr_session * session,
                tr_tracker * tracker )
 {
-    struct tr_tracker_request * req = createScrape( session, tracker );
+    struct tr_tracker_request * req;
+    assert( tr_isSession( session ) );
+
+    req = createScrape( session, tracker );
     tr_runInEventThread( session, invokeRequest, req );
 }
 
@@ -916,7 +934,10 @@ enqueueRequest( tr_session * session,
                 tr_tracker * tracker,
                 int          reqtype )
 {
-    struct tr_tracker_request * req = createRequest( session, tracker, reqtype );
+    struct tr_tracker_request * req;
+    assert( tr_isSession( session ) );
+
+    req = createRequest( session, tracker, reqtype );
     tr_runInEventThread( session, invokeRequest, req );
 }
 
@@ -928,7 +949,7 @@ trackerPulse( void * vsession )
     tr_torrent *               tor;
     const time_t               now = time( NULL );
 
-    if( !session->tracker )
+    if( !th )
         return FALSE;
 
     if( th->runningCount )
@@ -963,17 +984,16 @@ trackerPulse( void * vsession )
                 th->runningCount );
 
     /* free the tracker manager if no torrents are left */
-    if( ( session->tracker )
-      && ( session->tracker->runningCount < 1 )
-      && ( tr_sessionCountTorrents( session ) == 0 ) )
+    if(    ( th != NULL )
+        && ( th->shutdownHint != FALSE )
+        && ( th->runningCount < 1 )
+        && ( tr_sessionCountTorrents( session ) == 0 ) )
     {
-        tr_trackerSessionClose( session );
+        tr_trackerSessionDestroy( session );
+        return FALSE;
     }
 
-    /* if there are still running torrents (as indicated by
-     * the existence of the tracker manager) then keep the
-     * trackerPulse() timer alive */
-    return session->tracker != NULL;
+    return TRUE;
 }
 
 static void
@@ -1035,8 +1055,6 @@ tr_trackerNew( const tr_torrent * torrent )
     const tr_info * info = &torrent->info;
     tr_tracker *    t;
 
-    ensureGlobalsExist( torrent->session );
-
     t = tr_new0( tr_tracker, 1 );
     t->publisher                = TR_PUBLISHER_INIT;
     t->session                  = torrent->session;
