@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: transmission.h 7722 2009-01-16 17:31:10Z charles $
+ * $Id: transmission.h 8144 2009-04-05 14:20:13Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -147,7 +147,7 @@ static TR_INLINE tr_bool tr_isEncryptionMode( tr_encryption_mode m )
 #define TR_DEFAULT_RPC_WHITELIST "127.0.0.1"
 #define TR_DEFAULT_RPC_PORT_STR "9091"
 #define TR_DEFAULT_PEER_PORT_STR "51413"
-#define TR_DEFAULT_PEER_SOCKET_TOS_STR "8"
+#define TR_DEFAULT_PEER_SOCKET_TOS_STR "0"
 #define TR_DEFAULT_PEER_LIMIT_GLOBAL_STR "240"
 #define TR_DEFAULT_PEER_LIMIT_TORRENT_STR "60"
 
@@ -249,9 +249,9 @@ void tr_sessionLoadSettings( struct tr_benc  * dictionary,
  * @param dictionary
  * @see tr_sessionLoadSettings()
  */
-void tr_sessionSaveSettings( tr_session     * session,
-                             const char     * configDir,
-                             struct tr_benc * dictonary );
+void tr_sessionSaveSettings( tr_session             * session,
+                             const char             * configDir,
+                             const struct tr_benc *   dictonary );
 
 /**
  * Initialize a libtransmission session.
