@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: web.c 7873 2009-02-11 16:07:45Z charles $
+ * $Id: web.c 8204 2009-04-10 17:34:25Z charles $
  */
 
 #include <assert.h>
@@ -26,6 +26,13 @@
 #include "utils.h"
 #include "web.h"
 
+/* Use curl_multi_socket_action() instead of curl_multi_perform()
+   if libcurl >= 7.18.2.  See http://trac.transmissionbt.com/ticket/1844 */
+#if LIBCURL_VERSION_NUM >= 0x071202
+    #define USE_CURL_MULTI_SOCKET_ACTION
+#endif
+
+
 enum
 {
     /* arbitrary number */
@@ -65,6 +72,7 @@ struct tr_web
     CURLM * multi;
     tr_session * session;
     struct event timer_event;
+    tr_list * easy_queue;
     tr_list * fds;
 };
 
@@ -207,7 +215,10 @@ addTask( void * vtask )
         else /* don't set encoding on webseeds; it messes up binary data */
             curl_easy_setopt( easy, CURLOPT_ENCODING, "" );
 
-        {
+        if( web->still_running >= MAX_CONCURRENT_TASKS ) {
+            tr_list_append( &web->easy_queue, easy );
+            dbgmsg( ">> enqueueing a task... size is now %d", tr_list_size( web->easy_queue ) );
+        } else {
             const CURLMcode mcode = curl_multi_add_handle( web->multi, easy );
             tr_assert( mcode == CURLM_OK, "curl_multi_add_handle() failed: %d (%s)", mcode, curl_multi_strerror( mcode ) );
             if( mcode == CURLM_OK )
@@ -317,6 +328,27 @@ restart_timer( tr_web * g )
     evtimer_add( &g->timer_event, &interval );
 }
 
+static void
+add_tasks_from_queue( tr_web * g )
+{
+    while( ( g->still_running < MAX_CONCURRENT_TASKS ) 
+        && ( tr_list_size( g->easy_queue ) > 0 ) )
+    {
+        CURL * easy = tr_list_pop_front( &g->easy_queue );
+        if( easy )
+        {
+            const CURLMcode rc = curl_multi_add_handle( g->multi, easy );
+            if( rc != CURLM_OK )
+                tr_err( "%s", curl_multi_strerror( rc ) );
+            else {
+                dbgmsg( "pumped the task queue, %d remain",
+                        tr_list_size( g->easy_queue ) );
+                ++g->still_running;
+            }
+        }
+    }
+}
+
 static void
 web_close( tr_web * g )
 {
@@ -336,7 +368,7 @@ web_close( tr_web * g )
    and no tasks remain.  callers must not reference their g pointer
    after calling this function */
 static void
-tr_multi_perform( tr_web * g )
+tr_multi_perform( tr_web * g, int fd )
 {
     int closed = FALSE;
     CURLMcode mcode;
@@ -345,17 +377,28 @@ tr_multi_perform( tr_web * g )
             g->prev_running, g->still_running );
 
     /* invoke libcurl's processing */
+#ifdef USE_CURL_MULTI_SOCKET_ACTION
+    do {
+        dbgmsg( "calling curl_multi_socket_action..." );
+        mcode = curl_multi_socket_action( g->multi, fd, 0, &g->still_running );
+        fd = CURL_SOCKET_TIMEOUT;
+        dbgmsg( "done calling curl_multi_socket_action..." );
+    } while( mcode == CURLM_CALL_MULTI_SOCKET );
+#else
     do {
         dbgmsg( "calling curl_multi_perform..." );
         mcode = curl_multi_perform( g->multi, &g->still_running );
         dbgmsg( "done calling curl_multi_perform..." );
     } while( mcode == CURLM_CALL_MULTI_PERFORM );
+#endif
     tr_assert( mcode == CURLM_OK, "curl_multi_perform() failed: %d (%s)", mcode, curl_multi_strerror( mcode ) );
     if( mcode != CURLM_OK )
         tr_err( "%s", curl_multi_strerror( mcode ) );
 
     remove_finished_tasks( g );
 
+    add_tasks_from_queue( g );
+
     if( !g->still_running ) {
         assert( tr_list_size( g->fds ) == 0 );
         stop_timer( g );
@@ -371,9 +414,9 @@ tr_multi_perform( tr_web * g )
 
 /* libevent says that sock is ready to be processed, so wake up libcurl */
 static void
-event_cb( int fd UNUSED, short kind UNUSED, void * g )
+event_cb( int fd, short kind UNUSED, void * g )
 {
-    tr_multi_perform( g );
+    tr_multi_perform( g, fd );
 }
 
 /* libevent says that timer_ms have passed, so wake up libcurl */
@@ -381,7 +424,7 @@ static void
 timer_cb( int socket UNUSED, short action UNUSED, void * g )
 {
     dbgmsg( "libevent timer is done" );
-    tr_multi_perform( g );
+    tr_multi_perform( g, CURL_SOCKET_TIMEOUT );
 }
 
 /* CURLMOPT_SOCKETFUNCTION */
